'use strict';

const server = require("./setup");
const assert = require('chai').assert;
const hl = require('highland');
const db = require('tm-apps-db')(require('config').db);
const R = require('ramda');
const faker = require('../faker');
let utils = require("./utils");
const config = require('config');
const imageService = config.imageService;

const validators = utils.createValidators();

describe('Article Component', () => {

    before(function (done) {
        this.timeout(6000);
        db.sequelize.sync({force: true}).then(function () {
            done();
        });
    });
    after(() => {
        db.sequelize.close();
    });

    beforeEach(done => {
        hl(utils.given(db, 'section', faker.section71))
            .toCallback(done);
    });

    describe('basic article', () => {

        before(done => {
            hl(db.article.upsert(faker.article))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => {
                    return hl([
                        hl(utils.addArticleContent([faker.articleContent[0]], articleInstance)),
                        hl(utils.addArticleContent([faker.articleContent[2]], articleInstance)),
                        hl(utils.addTags(db, faker.tags, articleInstance))
                    ])
                        .merge()
                        .collect();
                })
                .toCallback(done);
        });

        before(done => {
            hl(db.publication.upsert(faker.publication))
            .toCallback(done);
        });

        it('gets a basic article', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {

                    const tagNames = R.pluck('name', res.body.data.attributes.searchTags);
                    const tagIds = R.pluck('tag_id', res.body.data.attributes.searchTags);

                    assert.equal(res.header['content-type'], 'application/json; charset=utf-8');
                    validators.news(res.body);
                    assert.strictEqual(res.body.data.id, 123456789);
                    assert.isTrue(R.is(Object, res.body.data.attributes));
                    assert.deepEqual(res.body.data.attributes.homeSection, {name: 'new section'});
                    assert.isNull(res.body.data.attributes.leadMedia);
                    assert.strictEqual(R.contains('Murder', res.body.data.attributes.tags), true);
                    assert.strictEqual(R.contains('Crime', res.body.data.attributes.tags), true);
                    assert.strictEqual(R.contains('Queen', res.body.data.attributes.tags), true);
                    assert.strictEqual('Queen', res.body.data.attributes.tags[0]);
                    assert.strictEqual(res.body.data.attributes.tags.length, 3);
                    assert.strictEqual(res.body.data.attributes.socialHeadline, 'Social headline');
                    assert.strictEqual(res.body.data.links.url, 'http://www.mirror.co.uk/incoming/file.ece?widgetId=4705627');
                    assert.strictEqual(res.body.data.links.offlineUrl, 'http://www.mirror.co.uk/incoming/file.ece?widgetId=5478357&service=tablet');
                    assert.strictEqual(res.body.included[0].data.attributes.body, 'This is the first paragraph');
                    assert.strictEqual(res.body.included[1].data.type, 'embed');
                    assert.strictEqual(res.body.included[1].data.attributes.subType, 'poll');
                    assert.strictEqual(res.body.included[1].data.id, 12345);
                    assert.strictEqual(res.body.data.attributes.lastModifiedDate, "2016-06-09T07:50:11.000Z");
                    assert.strictEqual(res.body.data.attributes.checksum, "2016-06-09T07:50:11.000Z");
                    assert.isObject(res.body.data.attributes.searchTags[0]);
                    assert.isTrue(R.contains(3, tagIds));
                    assert.isTrue(R.contains(33, tagIds));
                    assert.isTrue(R.contains('Crime', tagNames));
                    assert.isTrue(R.contains('Murder', tagNames));

                    done(err);
                });
        });
    });

    describe('Basic article with authors', () => {

        before(done => {
            hl([
                utils.given(db, 'article_author', faker.article_author)
            ])
                .merge()
                .collect()
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .toCallback(done);
        });

        it('returns the authors correctly', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.equal(res.body.data.attributes.authors.length, 1);
                    assert.equal(res.body.data.attributes.authors[0].name, 'Joe Bloggs');
                    assert.equal(res.body.data.attributes.authors[0].id, 12345);
                    done(err);
                });
        });
    });

    describe('failures', () => {
        it('404s gracefully when article doesn\'t exist', (done) => {
            server
                .get('/nationals/2/articles/1234566')
                .expect(404)
                .end((err, res) => {
                    assert.equal(res.body.code, 'NotFoundError');
                    assert.equal(res.body.message, 'nationals/2/articles/1234566 not available');
                    done(err);
                });
        });

        before(done => {
            const noPublicationArticle = R.merge(faker.article, {
                id: 'nationals.558008',
                escenic_id: 58008,
                home_section_id: 'nationals.558008'
            });
            hl(hl(db.article.upsert(noPublicationArticle)))
                .toCallback(done);
        });

        it('fails if section does not exist', (done) => {
            server
                .get('/nationals/2/articles/558008')
                .expect(404)
                .end((err, res) => {
                    assert.equal(res.body.code, 'NotFoundError');
                    assert.equal(res.body.message, 'Section nationals.558008 not available');
                    done();
                });
        });
    });

    describe("Last modified date reflects changes in other associated content types", () => {

        describe("Images (as they are split out)", () => {

            const fakeImageTimeStamps = {
                "last_modified_date": "2016-10-05 15:27:28.123000Z"
            };

            const fakeArticleTimeStamps = {
                "last_modified_date": "2016-10-05 13:27:28.804000Z"
            };

            const fakeImage = R.merge(faker.image, fakeImageTimeStamps);
            const fakeArticle = R.merge(faker.article, fakeArticleTimeStamps);


            before(done => {
                hl(utils.upsertContent(db, 'image', fakeImage))
                    .flatMap(() => hl(db.article.upsert(fakeArticle)))
                    .flatMap(() => hl(db.article_content.destroy({
                        where: {
                            article_id: faker.article.id
                        }
                    })))
                    .flatMap(() => hl(db.article.findById(faker.article.id)))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: faker.image.id,
                            escenic_id: faker.image.escenic_id,
                            source: faker.image.source,
                            type: 'image',
                            order_index: 2
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with an image', (done) => {
                server
                    .get('/nationals/2/articles/123456789')
                    .end((err, res) => {
                        validators.news(res.body);
                        assert.equal(res.body.data.attributes.lastModifiedDate, "2016-10-05T15:27:28.123Z");
                        done(err);
                    });

            });
        });

        describe('Videos (as they are not split out... yet)', () => {

            const galleryArticle = R.merge(faker.article, {
                id: 'nationals.82107343',
                "last_modified_date": "2016-10-05 13:27:28.804000Z",
                escenic_id: 82107343
            });

            const video2 = R.merge(faker.video, {
                id: 'nationals.6662383',
                escenic_id: 6662383,
                sub_type: 'brightcoveVideo',
                last_modified_date: "2016-10-05 15:27:28.123000Z"
            });

            before(done => {
                hl(utils.upsertContent(db, 'video', video2))
                    .flatMap(() => hl(utils.upsertContent(db, 'video', faker.video)))
                    .flatMap(() => hl(db.article.upsert(galleryArticle)))
                    .flatMap(() => hl(db.article.findById(galleryArticle.id)))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: video2.id,
                            escenic_id: video2.escenic_id,
                            source: video2.source,
                            type: 'video',
                            order_index: 3
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with a video', (done) => {
                server
                    .get('/nationals/2/articles/82107343')
                    .end((err, res) => {
                        validators.news(res.body);
                        assert.equal(res.body.data.attributes.lastModifiedDate, "2016-10-05T15:27:28.123Z");
                        done(err);
                    });
            });
        });

    });

    describe('Article with image in articleContent', () => {

        before(done => {
            hl(utils.upsertContent(db, 'image', faker.image))
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .flatMap(() => hl(db.article_content.destroy({
                    where: {
                        article_id: faker.article.id
                    }
                })))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        content_item_id: faker.image.id,
                        escenic_id: faker.image.escenic_id,
                        source: faker.image.source,
                        type: 'image',
                        order_index: 2
                    }
                ])))
                .toCallback(done);
        });

        it('gets a simple article with an image', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.equal(res.body.included[0].data.attributes.alternates.length, 4);
                    assert.isTrue(R.all(R.propSatisfies(R.test(/^.+b$/), 'crop'), res.body.included[0].data.attributes.alternates));
                    assert.equal(res.body.included[0].data.links.url, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510b/test-title`);
                    assert.equal(res.body.included[0].data.type, 'image');
                    assert.equal(res.body.included[0].data.attributes.width, 510);
                    assert.equal(res.body.included[0].data.attributes.caption, 'test caption');
                    done(err);
                });

        });
    });

    describe('Article with image static content', () => {

        before(done => {
            hl(utils.upsertContent(db, 'image', faker.image2))
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .flatMap(() => hl(db.article_content.destroy({
                    where: {
                        article_id: faker.article.id
                    }
                })))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        'content_item_id': faker.image2.id,
                        'escenic_id': faker.image2.escenic_id,
                        'source': faker.image2.source,
                        'type': 'image',
                        'order_index': 2,
                        'static_content': {
                            'caption': 'an article specific caption',
                            'alttext': 'an article specific alttext'
                        }
                    }
                ])))
                .toCallback(done);
        });

        it('gets an article with an image that also has an article relation', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.equal(res.body.included[0].data.links.url, `https://${imageService}.mirror.co.uk/incoming/article238989999.ece/ALTERNATES/s510b/test-title`);
                    assert.equal(res.body.included[0].data.type, 'image');
                    assert.equal(res.body.included[0].data.attributes.width, 510);
                    assert.equal(res.body.included[0].data.attributes.caption, 'an article specific caption');
                    assert.equal(res.body.included[0].data.attributes.altText, 'an article specific alttext');
                    done(err);
                });

        });
    });

    describe('Article with missing image in articleContent', () => {

        before(done => {
            hl(db.article.upsert(faker.article))
                .flatMap(() => hl(db.article_content.destroy({
                    where: {
                        'article_id': faker.article.id
                    }
                })))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        'content_item_id': faker.image.id + '1',
                        'static_content': {caption: 'a caption'},
                        'escenic_id': faker.image.escenic_id + 1,
                        'source': faker.image.source,
                        'type': 'image',
                        'order_index': 0
                    },
                    {
                        'static_content': {
                            'type': 'text',
                            'body': 'Paragraph 1'
                        },
                        'order_index': 1
                    }
                ])))
                .toCallback(done);
        });

        it('gets a simple article with a missing image', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.strictEqual(res.body.included.length, 1);
                    assert.strictEqual(res.body.included[0].data.type, 'text');
                    done(err);
                });

        });
    });

    describe('Article with lead media image', () => {

        before(done => {
            hl(utils.upsertContent(db, 'image', faker.image))
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        content_item_id: faker.image.id,
                        escenic_id: faker.image.escenic_id,
                        source: faker.image.source,
                        type: 'image',
                        order_index: 2,
                        static_content: {
                            caption: 'an article specific caption'
                        }
                    }
                ])))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: faker.article.id,
                    content_item_id: faker.image.id,
                    order_index: 0,
                    relation_type: 'picturerel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .toCallback(done);
        });

        it('gets an article with an image that also has a picturerel relation', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    const leadMedia = res.body.data.attributes.leadMedia.data;
                    assert.equal(leadMedia.type, 'image');
                    assert.equal(leadMedia.id, 6662380);
                    assert.equal(leadMedia.attributes.caption, 'an article specific caption');
                    assert.equal(leadMedia.attributes.title, 'test-title');
                    assert.equal(leadMedia.attributes.alternates[0].crop, 's510');
                    assert.equal(leadMedia.attributes.alternates[1].crop, 's615');
                    assert.equal(leadMedia.links.thumbnailUrl, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s170/test-title`);
                    done(err);
                });

        });
    });

    describe('Article with videos', () => {

        const galleryArticle = R.merge(faker.article, {
            id: 'nationals.82107342',
            escenic_id: 82107342
        });

        const video2 = R.merge(faker.video, {
            id: 'nationals.6662381',
            escenic_id: 6662381,
            sub_type: 'brightcoveVideo'
        });

        before(done => {
            hl(utils.upsertContent(db, 'video', video2))
                .flatMap(() => hl(utils.upsertContent(db, 'video', faker.video)))
                .flatMap(() => hl(db.article.upsert(galleryArticle)))
                .flatMap(() => hl(db.article.findById(galleryArticle.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        content_item_id: faker.video.id,
                        escenic_id: faker.video.escenic_id,
                        source: faker.video.source,
                        type: 'video',
                        order_index: 2,
                        static_content: {
                            title: 'an article specific title to a video'
                        }
                    },
                    {
                        content_item_id: video2.id,
                        escenic_id: video2.escenic_id,
                        source: video2.source,
                        type: 'video',
                        order_index: 3
                    }
                ])))
                .toCallback(done);
        });

        it('basic video in article-content', (done) => {
            server
                .get('/nationals/2/articles/82107342')
                .end((err, res) => {
                    validators.news(res.body);

                    const video1 = res.body.included[0];
                    const video2 = res.body.included[1];

                    assert.equal(res.body.included.length, 2);
                    assert.equal(video1.data.id, 4328934);
                    assert.equal(video1.data.type, 'video');
                    assert.equal(video1.data.attributes.title, 'an article specific title to a video');
                    assert.equal(video1.data.attributes.videoId, 'nf1j345131l3k451');
                    assert.equal(video1.data.attributes.subType, 'youtubeVideo');
                    assert.equal(video1.data.links.screenshotUrl, 'http://www.blahblah.co.uk/thumb_url');
                    assert.equal(video2.data.id, 6662381);
                    assert.equal(video2.data.type, 'video');
                    assert.equal(video2.data.attributes.title, 'test-title');
                    assert.equal(video2.data.attributes.videoId, 'nf1j345131l3k451');
                    assert.equal(video2.data.attributes.subType, 'brightcoveVideo');
                    assert.equal(video2.data.links.screenshotUrl, 'http://www.blahblah.co.uk/thumb_url');
                    done(err);
                });
        });
    });

    describe('Article with missing video in articleContent', () => {

        before(done => {
            hl(db.article.upsert(faker.article))
                .flatMap(() => hl(db.article_content.destroy({
                    where: {
                        'article_id': faker.article.id
                    }
                })))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([
                    {
                        'content_item_id': faker.video.id + '1',
                        'static_content': {title: 'a title'},
                        'escenic_id': faker.video.escenic_id + 1,
                        'source': faker.video.source,
                        'type': 'video',
                        'order_index': 0
                    },
                    {
                        'static_content': {
                            'type': 'text',
                            'body': 'Paragraph 1'
                        },
                        'order_index': 1
                    }
                ])))
                .toCallback(done);
        });

        it('gets a simple article with a missing video', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.strictEqual(res.body.included.length, 1);
                    assert.strictEqual(res.body.included[0].data.type, 'text');
                    done(err);
                });

        });
    });

    describe('Article with lead media video', () => {

        const article = R.merge(faker.article, {
            id: 'nationals.82107342',
            escenic_id: 82107342
        });

        const video2 = R.merge(faker.video, {
            id: 'nationals.6662381',
            escenic_id: 6662381,
            sub_type: 'brightcoveVideo'
        });

        before(done => {
            hl(utils.upsertContent(db, 'video', video2))
                .flatMap(() => hl(utils.upsertContent(db, 'video', faker.video)))
                .flatMap(() => hl(utils.upsertContent(db, 'image', faker.image)))
                .flatMap(() => hl(db.article.upsert(article)))
                .flatMap(() => hl(db.article.findById(article.id)))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: article.id,
                    content_item_id: faker.image.id,
                    order_index: 0,
                    relation_type: 'picturerel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: article.id,
                    content_item_id: faker.video.id,
                    order_index: 0,
                    relation_type: 'videorel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .toCallback(done);
        });

        it('gets an article with a lead video with an image', (done) => {
            server
                .get('/nationals/2/articles/82107342')
                .end((err, res) => {
                    validators.news(res.body);
                    const leadMedia = res.body.data.attributes.leadMedia.data;
                    assert.equal(leadMedia.type, 'video');
                    assert.equal(leadMedia.id, 4328934);
                    assert.equal(leadMedia.attributes.subType, 'youtubeVideo');
                    assert.equal(leadMedia.attributes.caption, 'an article specific caption');
                    assert.equal(leadMedia.links.screenshotUrl, 'http://www.blahblah.co.uk/thumb_url');
                    assert.equal(leadMedia.attributes.leadMedia.data.type, 'image');
                    assert.equal(leadMedia.attributes.leadMedia.data.id, 6662380);
                    assert.equal(leadMedia.attributes.leadMedia.data.attributes.caption, 'an article specific caption');
                    assert.equal(leadMedia.attributes.leadMedia.data.attributes.title, 'test-title');
                    assert.equal(leadMedia.attributes.leadMedia.data.links.thumbnailUrl, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s170/test-title`);
                    done(err);
                });

        });
    });

    describe('Article with lead media video with no image', () => {

        before(done => {
            hl(utils.upsertContent(db, 'video', faker.video))
                .flatMap(() => hl(utils.upsertContent(db, 'image', faker.image)))
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(() => hl(db.article_relation.destroy({
                    where: {
                        article_id: faker.article.id,
                        content_item_id: faker.image.id
                    }
                })))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: faker.article.id,
                    content_item_id: faker.video.id,
                    order_index: 0,
                    relation_type: 'videorel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .toCallback(done);
        });

        it('gets an article with a lead video that has a null lead media', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    const leadMedia = res.body.data.attributes.leadMedia.data;
                    assert.equal(leadMedia.type, 'video');
                    assert.equal(leadMedia.id, 4328934);
                    assert.equal(leadMedia.attributes.subType, 'youtubeVideo');
                    assert.equal(leadMedia.attributes.caption, 'an article specific caption');
                    assert.equal(leadMedia.links.screenshotUrl, 'http://www.blahblah.co.uk/thumb_url');
                    assert.equal(leadMedia.attributes.leadMedia, null);
                    done(err);
                });

        });
    });

    describe('Article with lead media video with a missing lead image', () => {

        before(done => {
            hl(utils.upsertContent(db, 'video', faker.video))
                .flatMap(() => hl(db.image.destroy({where: {id: faker.image.id}})))
                .flatMap(() => hl(db.article.upsert(faker.article)))
                .flatMap(() => hl(db.article_relation.destroy({
                    where: {
                        article_id: faker.article.id,
                        content_item_id: faker.image.id
                    }
                })))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: faker.article.id,
                    content_item_id: faker.video.id,
                    order_index: 0,
                    relation_type: 'videorel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: faker.article.id,
                    content_item_id: faker.image.id,
                    order_index: 0,
                    relation_type: 'picturerel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })))
                .toCallback(done);
        });

        it('gets an article with a lead video that has a missing image', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    const leadMedia = res.body.data.attributes.leadMedia.data;
                    assert.equal(leadMedia.type, 'video');
                    assert.equal(leadMedia.id, 4328934);
                    assert.equal(leadMedia.attributes.subType, 'youtubeVideo');
                    assert.equal(leadMedia.attributes.caption, 'an article specific caption');
                    assert.equal(leadMedia.links.screenshotUrl, 'http://www.blahblah.co.uk/thumb_url');
                    assert.equal(leadMedia.attributes.leadMedia, null);
                    done(err);
                });

        });
    });

    describe('ArticleContent order', () => {

        const paragraphs = [1, 4, 2, 5, 0, 3].map(order_number => {
            return {
                static_content: {
                    type: 'text',
                    body: `Paragraph ${order_number}`
                },
                order_index: order_number
            };
        });

        paragraphs[0] = {
            order_index: 1,
            article_id: 'nationals.123456789',
            content_item_id: 'nationals.6662380'
        };

        before(done => {
            hl(db.article.upsert(faker.article))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(articleInstance.purgeContent()))
                .flatMap(() => hl(utils.upsertContent(db, 'image', faker.image)))
                .flatMap(() => hl(db.article.findById(faker.article.id)))
                .flatMap(articleInstance => hl(utils.addArticleContent(paragraphs, articleInstance)))
                .toCallback(done);
        });

        it('check the order updates', (done) => {
            server
                .get('/nationals/2/articles/123456789')
                .end((err, res) => {
                    validators.news(res.body);
                    const included = res.body.included;
                    assert.equal(included[0].data.attributes.body, 'Paragraph 0');
                    assert.equal(included[1].data.type, 'image');
                    assert.equal(included[2].data.attributes.body, 'Paragraph 2');
                    assert.equal(included[3].data.attributes.body, 'Paragraph 3');
                    assert.equal(included[4].data.attributes.body, 'Paragraph 4');
                    assert.equal(included[5].data.attributes.body, 'Paragraph 5');

                    done(err);
                });

        });
    });

    describe('Article with gallery', () => {

        const imageIds = ['1181181', '1181182', '1181183'];

        const images = imageIds.map(id => {
            return R.merge(faker.image, {
                id: `nationals.${id}`,
                escenic_id: Number(id)
            });
        });

        const article = R.merge(faker.article, {
            id: 'nationals.118118',
            escenic_id: 118118
        });

        const articleContent = {
            order_index: 0,
            content_item_id: faker.gallery.id
        };

        const galleryContent = imageIds.map((id, i) => {
            return {
                order_index: i,
                image_id: `nationals.${id}`
            };
        });

        before(done => {
            hl(images.map(image => {
                return utils.upsertContent(db, 'image', image);
            }))
                .collect()
                .toCallback(done);
        });

        before(done => {
            hl(db.article.upsert(article))
                .flatMap(() => hl(db.article.findById(article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([articleContent])))
                .toCallback(done);
        });

        before(done => {
            hl(utils.upsertContent(db, 'gallery', faker.gallery))
                .flatMap(() => hl(db.gallery.findById(faker.gallery.id)))
                .flatMap(galleryInstance => hl(galleryInstance.setContent(galleryContent)))
                .toCallback(done);
        });

        it('gets an article that contains a gallery with 1 lead image and 2 included images', (done) => {
            server
                .get('/nationals/2/articles/118118')
                .end((err, res) => {
                    const article = res.body;
                    validators.news(article);
                    const gallery = article.included[0];
                    assert.equal(R.path(['data', 'type'], gallery), 'gallery');
                    assert.equal(R.path(['data', 'id'], gallery), 1181180);
                    assert.equal(R.path(['data', 'attributes', 'title'], gallery), 'test-title');
                    assert.equal(R.path(['data', 'attributes', 'leadMedia', 'data', 'id'], gallery), 1181181);
                    assert.equal(R.path(['data', 'id'], gallery.included[0]), 1181182);
                    assert.equal(R.path(['data', 'type'], gallery.included[0]), 'image');
                    assert.equal(R.path(['data', 'attributes', 'alternates', 'length'], gallery.included[0]), 1);
                    assert.deepEqual(R.path(['data', 'attributes', 'alternates'], gallery.included[0]),  [
                        {
                            crop: 's1227b',
                            height: 1000,
                            width: 1227
                        }
                    ]);
                    assert.equal(R.path(['data', 'id'], gallery.included[1]), 1181183);
                    assert.equal(R.path(['data', 'type'], gallery.included[1]), 'image');
                    done(err);
                });
        });
    });

    describe('Article with missing gallery', () => {

        const article = R.merge(faker.article, {
            'id': 'nationals.118118',
            'escenic_id': 118118
        });

        const articleContent = [
            {
                static_content: {
                    type: 'text',
                    body: 'some text'
                },
                order_index: 0
            },
            {
                order_index: 1,
                content_item_id: faker.gallery.id
            }
        ];

        before(done => {
            hl(db.article.upsert(article))
                .flatMap(() => hl(db.content_item.destroy({
                    where: {
                        gallery_id: faker.gallery.id
                    }
                })))
                .flatMap(() => hl(db.article_content.destroy({
                    where: {
                        article_id: article.id
                    }
                })))
                .flatMap(() => hl(db.article.findById(article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent(articleContent)))
                .toCallback(done);
        });

        it('gets an article that contains a gallery with 1 lead image and 2 included images', (done) => {
            server
                .get('/nationals/2/articles/118118')
                .end((err, res) => {
                    validators.news(res.body);
                    assert.strictEqual(res.body.included.length, 1);
                    assert.strictEqual(res.body.included[0].data.type, 'text');
                    done(err);
                });
        });
    });

    describe('Article with lead media gallery', () => {

        const imageIds = ['1181181', '1181182', '1181183'];

        const images = imageIds.map(id => {
            return R.merge(faker.image, {
                id: `nationals.${id}`,
                escenic_id: Number(id)
            });
        });

        const article = R.merge(faker.article, {
            id: 'nationals.118118',
            escenic_id: 118118
        });

        const articleContent = {
            order_index: 0,
            content_item_id: faker.gallery.id
        };

        const galleryContent = imageIds.map((id, i) => {
            return {
                order_index: i,
                image_id: `nationals.${id}`
            };
        });

        before(done => {
            hl(images.map(image => {
                return utils.given(db, 'image', image);
            }))
                .collect()
                .toCallback(done);
        });

        before(done => {
            hl(db.article.upsert(article))
                .flatMap(() => hl(db.article.findById(article.id)))
                .flatMap(articleInstance => hl(articleInstance.setContent([articleContent])))
                .toCallback(done);
        });

        before(done => {
            hl(utils.upsertContent(db, 'gallery', faker.gallery))
                .flatMap(() => hl(db.gallery.findById(faker.gallery.id)))
                .flatMap(galleryInstance => hl(galleryInstance.setContent(galleryContent)))
                .toCallback(done);
        });

        before(done => {
            hl([
                db.article_relation.upsert({
                    article_id: 'nationals.118118',
                    content_item_id: faker.video.id,
                    order_index: 0,
                    relation_type: 'videorel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                }),
                db.article_relation.upsert({
                    article_id: 'nationals.118118',
                    content_item_id: faker.gallery.id,
                    order_index: 0,
                    relation_type: 'picturerel',
                    static_content: {
                        caption: 'an article specific caption'
                    }
                })
            ])
                .flatMap(hl)
                .collect()
                .toCallback(done);
        });

        it('gets an article with a gallery lead media', (done) => {
            server
                .get('/nationals/2/articles/118118')
                .end((err, res) => {
                    const leadMedia = res.body.data.attributes.leadMedia;
                    assert.equal(leadMedia.data.type, 'gallery');
                    done(err);
                });

        });
    });

    describe('Article with html', () => {

        before(done => {
            hl(db.publication.upsert(faker.publication))
            .toCallback(done);
        });

        const htmlArticle = R.merge(faker.article, {
            id: 'nationals.1234567',
            escenic_id: 1234567
        });

        describe('in article-content', () => {
            before(done => {
                hl(utils.upsertContent(db, 'html', faker.html))
                    .flatMap(() => hl(db.article.upsert(htmlArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.1234567')))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: 'nationals.8392',
                            escenic_id: 8392,
                            source: 'nationals',
                            type: 'html',
                            order_index: 1
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with html embedded', done => {
                server
                    .get('/nationals/2/articles/1234567')
                    .end((err, res) => {
                        validators.news(res.body);

                        const htmls = res.body.included;
                        assert.equal(htmls.length, 1);

                        const html = htmls[0];

                        assert.equal(html.data.id, 8392);
                        assert.equal(html.data.type, 'embed');
                        assert.equal(html.data.attributes.subType, 'interactiveHTML');
                        assert.equal(html.data.attributes.label, 'Interactive Content');
                        assert.equal(html.data.links.url, 'http://www.mirror.co.uk/incoming/file.ece/embedded-webview/mirror-8392');
                        done(err);
                    });
            });
        });

        describe('missing html', () => {

            before(done => {
                hl(db.article_content.destroy({where: {article_id: 'nationals.1234567'}}))
                    .flatMap(() => hl(db.article.upsert(htmlArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.1234567')))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: 'nationals.8390',
                            escenic_id: 8390,
                            source: 'nationals',
                            type: 'html',
                            order_index: 1
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with html missing', done => {
                server
                    .get('/nationals/2/articles/1234567')
                    .end((err, res) => {
                        validators.news(res.body);
                        const htmls = res.body.included;
                        assert.equal(htmls.length, 0);
                        done(err);
                    });

            });
        });

        describe('tweet html in article-content', () => {
            before(done => {
                hl(db.article_content.destroy({where: {article_id: 'nationals.1234567'}}))
                    .flatMap(() => hl(db.article.upsert(htmlArticle)))
                    .flatMap(() => hl(utils.upsertContent(db, 'html', faker.tweetHtml)))
                    .flatMap(() => hl(utils.upsertContent(db, 'tweet', faker.tweet)))
                    .flatMap(() => hl(db.html_content_item.upsert({html_id: faker.tweetHtml.id, tweet_id: faker.tweet.id})))
                    .flatMap(() => hl(db.article.upsert(htmlArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.1234567')))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: faker.tweetHtml.id,
                            escenic_id: faker.tweetHtml.escenic_id,
                            source: 'nationals',
                            type: 'html',
                            order_index: 1
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with a tweet embedded', done => {
                server
                    .get('/nationals/2/articles/1234567')
                    .end((err, res) => {
                        const content = R.head(R.path(['body', 'included'], res));
                        assert.equal(content.data.type, 'tweet');
                        assert.equal(content.data.id, 1123581321345589);
                        assert.equal(content.data.attributes.tweet, 'Ed Balls');
                        done(err);
                    });
            });
        });
    });

    describe('Article with quiz', () => {

        const quizArticle = R.merge(faker.article, {
            id: 'nationals.24362456',
            escenic_id: 24362456
        });

        describe('quiz in article-content', () => {

            before(done => {
                hl(utils.upsertContent(db, 'quiz', faker.quiz))
                    .flatMap(() => hl(db.article.upsert(quizArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.24362456')))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: 'nationals.3425',
                            escenic_id: 3425,
                            source: 'nationals',
                            type: 'quiz',
                            order_index: 1
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with a quiz embedded', done => {
                server
                    .get('/nationals/2/articles/24362456')
                    .end((err, res) => {
                        assert.isTrue(res.body.data.attributes.webview);
                        assert.equal(res.body.included[0].data.type, "quiz");
                        assert.equal(res.body.included[0].data.id, 3425);
                        assert.equal(res.body.included[0].data.attributes.title, "Quiz Title");
                        done(err);
                    });
            });
        });
    });

    describe('Article with tweet', () => {
        const tweet = {
            id: '1234567890987654321',
            data: {
                title: 'Hello, World!',
                id_str: '1234567890987654321'
            }
        };

        const tweetArticle = R.merge(faker.article, {
            id: 'nationals.234567',
            escenic_id: 234567
        });

        describe('in article-content', () => {
            before(done => {
                hl(utils.upsertContent(db, 'tweet', tweet))
                    .flatMap(() => hl(db.article.upsert(tweetArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.234567')))
                    .flatMap(articleInstance => hl(articleInstance.purgeContent()).map(() => articleInstance))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: '1234567890987654321',
                            type: 'tweet',
                            order_index: 0
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with tweet embedded', done => {
                server
                    .get('/nationals/2/articles/234567')
                    .end((err, res) => {
                        validators.news(res.body);
                        const tweets = res.body.included;
                        assert.equal(tweets.length, 1);

                        const tweet = tweets[0];
                        assert.ok(tweet);
                        done(err);
                    });
            });
        });

        describe('missing tweet', () => {
            before(done => {
                hl(db.article_content.destroy({where: {article_id: 'nationals.234567'}}))
                    .flatMap(() => hl(db.article.upsert(tweetArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.234567')))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: '21334245234535345',
                            type: 'tweet',
                            order_index: 0
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with tweet missing', done => {
                server
                    .get('/nationals/2/articles/234567')
                    .end((err, res) => {
                        validators.news(res.body);
                        const tweets = res.body.included;
                        assert.equal(tweets.length, 0);
                        done(err);
                    });

            });
        });
    });

    describe('Article with instagram', () => {
        const instagram = {
            id: '7g8e8wf8w3',
            data: {
                caption: 'caption',
                username: 'someUser',
                url: 'https://www.instagram.com/p/7g8e8wf8w3',
                instagramId: '7g8e8wf8w3',
                image_url: "someUrl",
                profileImageUrl: "someProfile"
            }
        };

        const instagramArticle = R.merge(faker.article, {
            id: 'nationals.3453465',
            escenic_id: 3453465
        });
        describe('in article-content', () => {
            before(done => {
                hl(utils.upsertContent(db, 'instagram', instagram))
                    .flatMap(() => hl(db.article.upsert(instagramArticle)))
                    .flatMap(() => hl(db.article.findById('nationals.3453465')))
                    .flatMap(articleInstance => hl(articleInstance.purgeContent()).map(() => articleInstance))
                    .flatMap(articleInstance => hl(articleInstance.setContent([
                        {
                            content_item_id: '7g8e8wf8w3',
                            type: 'instagram',
                            order_index: 0
                        }
                    ])))
                    .toCallback(done);
            });

            it('gets a simple article with instagram embedded', done => {
                server
                    .get('/nationals/2/articles/3453465')
                    .end((err, res) => {
                        validators.news(res.body);
                        const instagrams = res.body.included;
                        assert.equal(instagrams.length, 1);
                        assert.equal(instagrams[0].data.type, "instagram");
                        assert.equal(instagrams[0].data.id, "7g8e8wf8w3");
                        assert.equal(instagrams[0].data.attributes.caption, "caption");
                        assert.equal(instagrams[0].data.attributes.instagramId, "7g8e8wf8w3");
                        assert.equal(instagrams[0].data.attributes.username, "someUser");
                        assert.equal(instagrams[0].data.attributes.imageUrl, "someUrl");
                        assert.equal(instagrams[0].data.attributes.profileImageUrl, "someProfile");
                        assert.equal(instagrams[0].data.links.url, "https://www.instagram.com/p/7g8e8wf8w3/embed");
                        done(err);
                    });
            });
        });
    });

    describe('Live event with all fields', () => {

        before(done => {

            hl(db.article.upsert(faker.liveEvent))
                .flatMap(() => hl(db.article.findById(faker.liveEvent.id)))
                .flatMap(articleInstance => utils.flatTaps([
                    articleInstance.purgeRelations(),
                    articleInstance.purgeAuthors(),
                    articleInstance.purgeContent()
                ], articleInstance))
                .flatMap(articleInstance => hl(db.article_author.upsert(faker.article_author_le)).map(R.always(articleInstance)))
                .flatMap(articleInstance => hl(utils.addTags(db, faker.tags, articleInstance)).map(R.always(articleInstance)))
                .flatMap(articleInstance => {
                    return hl([
                        hl(utils.addArticleContent([faker.articleContent[0]], articleInstance)),
                        hl(utils.addArticleContent([faker.articleContent[2]], articleInstance))
                    ])
                        .merge()
                        .collect();
                })
                .flatMap(() => hl(db.publication.upsert(faker.publication)))
                .toCallback(done);
        });

        it('gets a live event with all fields', (done) => {
            server
                .get('/nationals/2/articles/9014003')
                .end((err, res) => {
                    const attributes = res.body.data.attributes;

                    validators.liveEvent(res.body);
                    assert.strictEqual(res.body.data.type, 'live-event');
                    assert.strictEqual(res.body.data.id, 9014003);
                    assert.strictEqual(attributes.createdDate, '2016-10-10T00:05:01.000Z');
                    assert.strictEqual(attributes.publishedDate, '2016-10-10T00:05:01.000Z');
                    assert.strictEqual(attributes.lastModifiedDate, '2016-10-10T06:05:32.000Z');
                    assert.strictEqual(attributes.state, 'published');
                    assert.isFalse(attributes.sponsored);
                    assert.isFalse(attributes.advertorial);
                    assert.isTrue(attributes.webview);
                    assert.strictEqual(attributes.title, 'Fake Live Event');
                    assert.deepEqual(attributes.homeSection, {name: 'new section'});
                    assert.deepEqual(attributes.authors, [{name: 'Joe LiveEvent', type: 'author', id:12345}]);
                    assert.deepEqual(attributes.tags.sort(), ['Murder', 'Crime'].sort());
                    assert.isNull(attributes.leadMedia);

                    const links = res.body.data.links;
                    assert.strictEqual(links.ownerUrl, 'http://www.mirror.co.uk/live-event-9014003');
                    assert.strictEqual(links.url, 'http://www.mirror.co.uk/incoming/filename.ece?widgetId=4705627');
                    assert.strictEqual(links.offlineUrl, 'http://www.mirror.co.uk/incoming/filename.ece?widgetId=5478357&service=tablet');
                    assert.strictEqual(links.liveEventEntriesUrl, 'https://www.mirror.co.uk/incoming/filename.ece/embedded-webview/mirror-9014003');

                    const included = res.body.included;
                    assert.strictEqual(included[0].data.type, 'text');
                    assert.strictEqual(included[1].data.type, 'embed');
                    assert.strictEqual(included[2].data.type, 'liveEventEntries');
                    assert.strictEqual(included[2].data.links.url, 'https://www.mirror.co.uk/incoming/filename.ece/embedded-webview/mirror-9014003');

                    done(err);
                });
        });
    });

    describe('Live event with lead media image', () => {

        before(done => {
            utils.flatTap(utils.upsertContent(db, 'image', faker.image), hl(db.article.upsert(faker.liveEvent)))
                .flatMap(() => hl(db.article.findById(faker.liveEvent.id)))
                .flatMap(articleInstance => utils.flatTaps([articleInstance.purgeRelations(), articleInstance.purgeAuthors()], articleInstance))
                .flatMap(() => hl(db.article_relation.upsert({
                    article_id: faker.liveEvent.id,
                    content_item_id: faker.image.id,
                    order_index: 0,
                    relation_type: 'picturerel',
                    static_content: {
                        caption: 'a live event specific caption'
                    }
                })))
                .toCallback(done);
        });

        it('gets a live event which has a picturerel relation', (done) => {
            server
                .get('/nationals/2/articles/9014003')
                .end((err, res) => {
                    const leadMedia = res.body.data.attributes.leadMedia.data;
                    assert.strictEqual(leadMedia.type, 'image');
                    assert.strictEqual(leadMedia.id, 6662380);
                    assert.strictEqual(leadMedia.attributes.caption, 'a live event specific caption');
                    assert.strictEqual(leadMedia.attributes.title, 'test-title');
                    assert.strictEqual(leadMedia.links.thumbnailUrl, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s170/test-title`);
                    done(err);
                });

        });
    });
});

