'use strict';

const assert = require('chai').assert;
const expect = require('chai').expect;
const server = require('./setup');
const nock = require('nock');
const faker = require('../faker');
const config = require('../../config/default.json');


describe('Author Component Test : ', () => {

    describe('basic author', () => {

        nock(config.authorsEndpoint)
        .get('/')
        .query({ids:'nationals.58'})
        .reply(200, faker.author_api_response);

        nock(config.authorsEndpoint)
        .get('/')
        .query({ids:'nationals.78'})
        .reply(404, faker.author_api_response_404);
        
        nock(config.authorsEndpoint)
        .get('/')
        .query({ids:'nationals.88'})
        .reply(200, faker.author_api_response_no_links);

        nock(config.authorsEndpoint)
        .get('/')
        .query({ids:'nationals.99'})
        .reply(200, []);
        

        it('should reply with an author if the id exists', done => {
            server
                .get('/nationals/2/authors/58')
                .end((err, res) => {

                    // type and structure
                    expect(res.body).to.be.an('object');
                    expect(res.body).to.have.property('data');
                    expect(res.body.data).to.have.property('attributes');
                    expect(res.body.data).to.have.property('links');

                    const data = res.body.data;

                    // assertions
                    assert.strictEqual(data.type, 'author');
                    assert.strictEqual(data.id, 58);
                    assert.strictEqual(data.attributes.name, faker.author_api_response[0].name);
                    assert.strictEqual(data.attributes.description, faker.author_get_response.description);
                    assert.strictEqual(data.attributes.twitterId, faker.author_get_response.twitterId);
                    assert.strictEqual(data.links.imageUrl, faker.author_get_response.imageUrl);
                    assert.strictEqual(data.links.url, faker.author_get_response.url);

                    done();
                });
        });
        
        it('should reply gracefully if the id is malformed', done => {
            server
                .get('/nationals/2/authors/78')
                .end((err, res) => {

                    expect(res.body).to.be.an('object');
                    expect(res.body).to.have.property('message');
                    
                    assert.strictEqual(res.statusCode, 404);
                    assert.strictEqual(res.body.message, faker.author_api_response_404.message);

                    done();
                });
        });

        it('should reply gracefully if the id does not exist', done => {
            server
                .get('/nationals/2/authors/99')
                .end((err, res) => {

                    expect(res.body).to.be.an('object');
                    expect(res.body).to.have.property('message');
                    
                    assert.strictEqual(res.statusCode, 404);
                    assert.strictEqual(res.body.message, faker.author_api_response_404.message);

                    done();
                });
        });

        it('should not have links property if the author hasn\'t got links', done => {
            server
                .get('/nationals/2/authors/88')
                .end((err, res) => {

                    // type and structure
                    expect(res.body).to.be.an('object');
                    expect(res.body).to.have.property('data');
                    expect(res.body.data).to.have.property('attributes');
                    expect(res.body.data).to.not.have.property('links');

                    const data = res.body.data;

                    // assertions
                    assert.strictEqual(data.type, 'author');
                    assert.strictEqual(data.id, 88);
                    assert.strictEqual(data.attributes.name, faker.author_api_response[0].name);
                    assert.strictEqual(data.attributes.description, faker.author_get_response.description);
                    assert.strictEqual(data.attributes.twitterId, faker.author_get_response.twitterId);
                    
                    done();
                });
        });
    });
});