"use strict";

const fs = require('fs');
let hl = require("highland");
let R = require("ramda");
const path = require('path');
const Validator = require('jsonschema').Validator;

const flatTap = R.curry((xs, x) => {
    return hl(xs).collect().flatMap(() => hl.of(x));
});

module.exports = {
    "given": (db, model, precondition) => {
        return hl(db[model].upsert(precondition));
    },

    "addArticleContent": R.curry((staticContent, articleInstance) => {
        return hl([staticContent])
            .flatMap(x => hl(articleInstance.setContent(x)));
    }),

    "addTags": R.curry((db, tags, articleInstance) => {

        const actualTags = tags.map(tag => {
            return R.pipe(
                R.omit(["id"]),
                R.assoc("tag_id", tag.id)
            )(tag);
        });

        return hl(articleInstance.purgeTags())
            .flatMap(() => hl(db.tag.destroy({
                "where": {
                    "id": tags.map(tag => tag.id)
                }
            })))
            .flatMap(() => hl(db.tag.bulkCreate(tags)))
            .flatMap(() => hl(articleInstance.setTags(actualTags)));
    }),

    "upsertContent": (db, modelName, data) => {
        const id = data.id;
        return db[modelName].upsert(data)
            .then(() => db.content_item.upsert(R.assoc(`${modelName}_id`, id, {id: id})));
    },

    "flatTap": flatTap,

    "flatTaps": R.curry((xs, x) => {
        return R.reduce((prev, curr) => prev.flatMap(() => flatTap(curr, x)), hl.of([]), xs);
    }),

    "createValidators": () => {
        const schemaPath = path.resolve('schemas');
        const v = new Validator();
        return R.reduce((validators, file) => {
            const type = file.replace('.json', '');
            const schema = require(`${schemaPath}/${file}`);
            v.addSchema(schema, `/${type}`);
            validators[type] = instance => {
                const res = v.validate(instance, schema);
                if (!R.isEmpty(res.errors)) {
                    console.log(res.errors);
                    throw new Error('JSON Schema validation failed!');
                }
            };
            return validators;
        }, {}, R.filter(R.test(/\.json$/), fs.readdirSync(schemaPath)));
    }
};