'use strict';

const rewire = require('rewire');
const assert = require('chai').assert;
const getAuthors = rewire('../../../lib/get/authors');
const sinon = require("sinon");
let nock = require("nock");

const config = {
    "authorsEndpoint": "http://authors_url.com"
};

describe('Authors Request', () => {

    describe('requestAuthors', () => {

        it('handles failing request by simply returning the list of local author tuples', (done) => {

            const authors = [
                {
                    name: 'Gimme Credit'
                }
            ];

            const requestAuthors = getAuthors.__get__('requestAuthors');
            requestAuthors(config, authors, (authorsList) => {
                const actual = authorsList[0];
                assert.equal(actual[0].type, 'author');
                assert.equal(actual[1].name, 'Gimme Credit');
                done();
            });
        });

        it('if authors service returns empty array, should return the list of local author tuples', (done) => {

            nock('http://authors_url.com')
                .get('/')
                .reply(200, []);

            const authors = [
                {
                    name: 'Gimme Credit'
                }
            ];

            const requestAuthors = getAuthors.__get__('requestAuthors');
            requestAuthors(config, authors, (authorsList) => {
                const actual = authorsList[0];
                assert.equal(actual[0].type, 'author');
                assert.equal(actual[1].name, 'Gimme Credit');
                done();
            });
        });

        it('if authors service returns authors, should merge the results into local authors', (done) => {
            nock('http://authors_url.com')
                .get('/?ids=nationals.1234&ids=12345')
                .reply(200, [{
                    id: "nationals.1234",
                    url: 'author_url',
                    imageUrl: 'author_image_url'
                }]);

            const authors = [
                {
                    id: "nationals.1234",
                    name: "Gimme Credit"
                },
                {
                    id: 12345,
                    name: 'Morgan Credit'
                }
            ];

            const requestAuthors = getAuthors.__get__('requestAuthors');
            requestAuthors(config, authors, authorsList => {
                const first = authorsList[0];
                const second = authorsList[1];
                assert.equal(first[0].type, 'author');
                assert.equal(first[1].name, 'Gimme Credit');
                assert.equal(first[1].url, 'author_url');
                assert.equal(first[1].imageUrl, 'author_image_url');
                assert.equal(second[0].type, 'author');
                assert.equal(second[1].name, 'Morgan Credit');
                assert.equal(second[1].imageUrl, void 0);
                done();
            });
        });

        it('handles authors with ids and without ids simultaneously (only making requests for authors with ids)', (done) => {
            nock('http://authors_url.com')
                .get('/?ids=nationals.9999')
                .reply(200, [{
                    id: "nationals.9999",
                    url: 'author_url',
                    imageUrl: 'author_image_url'
                }]);

            const authors = [
                {
                    id: "nationals.9999",
                    name: "Paul Simon"
                },
                {
                    name: 'Art Garfunkel'
                }
            ];

            const requestAuthors = getAuthors.__get__('requestAuthors');
            requestAuthors(config, authors, authorsList => {
                const first = authorsList[0];
                const second = authorsList[1];
                assert.equal(first[0].type, 'author');
                assert.equal(first[1].name, 'Paul Simon');
                assert.equal(first[1].url, 'author_url');
                assert.equal(first[1].imageUrl, 'author_image_url');
                assert.equal(second[0].type, 'author');
                assert.equal(second[1].name, 'Art Garfunkel');
                assert.equal(second[1].imageUrl, void 0);
                done();
            });
        });

    });

    describe('groupAuthors', () => {

        const groupAuthors = getAuthors.__get__('groupAuthors');

        it('groups Authors by whether they have an id property',  () => {
            const authors = [{id: 1, name: 'alice'}, {id: 2, name: 'bob'}, {name: 'caroline'}, {id: 3, name: 'dan'}];
            const result = groupAuthors(authors);
            assert.deepEqual(result.withIds, [
                { id: 1, name: 'alice' },
                { id: 2, name: 'bob' },
                { id: 3, name: 'dan' }
            ]);
            assert.deepEqual(result.withoutIds, [
                { name: 'caroline' }
            ]);
        });

        it('returns an empty list for a empty group',  () => {
            const authors = [{id: 1, name: 'alice'}, {id: 2, name: 'bob'}, {id: 3, name: 'dan'}];
            const result = groupAuthors(authors);
            assert.deepEqual(result.withoutIds, []);
        });

        it('handles an empty authors array',  () => {
            const authors = [];
            const result = groupAuthors(authors);
            assert.deepEqual(result.withoutIds, []);
            assert.deepEqual(result.withIds, []);
        });
    });

    describe('export function returns a promise contains the resolved authors', () => {

        it("returns a promise", () => {

            const stub = sinon.stub().returns([{
                "name": "Gimme Credit",
                "author_id": "nationals.1",
                "id": "nationals.1"
            }]);

            getAuthors.__set__("requestAuthors", (config, authors, cb) => {
                cb(stub(authors));
            });

            const actual = getAuthors(config, [
                {
                    "dataValues": {
                        "name": "Gimme Credit",
                        "author_id": "nationals.1"
                    }
                }
            ]);
            return actual.then((res) => {
                assert.equal(res[0].name, "Gimme Credit");
                assert.equal(stub.firstCall.args[0][0].id, "nationals.1");
                assert.equal(stub.firstCall.args[0][0].author_id, "nationals.1");
                assert.equal(stub.firstCall.args[0][0].name, "Gimme Credit");
                assert.equal(res.length, 1);
            });
        });

        it('returns an empty array as authors doesn\'t exist', () => {
            getAuthors(config, []).then(actual => assert.deepEqual(actual, []));
        });
    });

});
