'use strict';

const assert = require('chai').assert;
const rewire = require('rewire');
const index = rewire('../../../lib/get');
const sinon = require('sinon');

describe('Get Requests Index', () => {


    describe('createParallelRequests()', () => {
        index.__set__('getAuthors', sinon.stub().returns('Authors'));
        index.__set__('getGalleries', sinon.stub().returns('Galleries'));
        index.__set__('getImages', sinon.stub().returns('Images'));
        index.__set__('getTags', sinon.stub().returns('Tags'));

        const getAuthors = index.__get__('getAuthors');
        const getGalleries = index.__get__('getGalleries');
        const getImages = index.__get__('getImages');
        const getTags = index.__get__('getTags');

        beforeEach(() => {
            getAuthors.reset();
            getGalleries.reset();
            getImages.reset();
            getTags.reset();
        });
        
        it('passes the article and other contentTypes', (done) => {
            index.__set__('groupByArticleRelations', sinon.stub().returns({
                gallery: [
                    'A gallery'
                ]
            }));
            const groupByArticleRelations = index.__get__('groupByArticleRelations');

            const article = {
                article_relations: ['article_relations'],
                article_contents: ['article_contents'],
                article_authors: ['article_authors'],
                get: () => 'news'
            };

            index('db', {}, article)
                .tap((res) => {
                    assert.deepEqual(res[0], [{type: 'news'}, article]);
                    assert.equal(res[1], 'Galleries');
                    assert.equal(res[2], 'Images');
                    assert.equal(res[3], 'Tags');
                    assert.equal(res[4], 'Authors');

                    assert.isTrue(groupByArticleRelations.calledOnce);
                    assert.deepEqual(groupByArticleRelations.firstCall.args[0],
                        [
                            'article_relations',
                            'article_contents'
                        ]
                    );

                    assert.isTrue(getGalleries.calledOnce);
                    assert.isTrue(getGalleries.calledWith('db', [
                        'A gallery'
                    ]));

                    assert.isTrue(getAuthors.calledOnce);
                    assert.deepEqual(getAuthors.firstCall.args[1],
                        [
                            'article_authors'
                        ]
                    );
                })
                .toCallback(done);
        });


    });

    describe('groupByArticleRelations()', () => {

        const groupByArticleRelations = index.__get__('groupByArticleRelations');

        it('should push images onto images list', () => {
            const relations = [
                {
                    content_item: {
                        dataValues: {
                            image_id: 1
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            image_id: 2
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            image_id: 3
                        }
                    }
                }
            ];

            const groupedRelations = groupByArticleRelations(relations);
            assert.deepEqual(groupedRelations.image[0], {
                dataValues: {
                    image_id: 1
                }
            });
            assert.deepEqual(groupedRelations.image[1], {
                dataValues: {
                    image_id: 2
                }
            });
            assert.deepEqual(groupedRelations.image[2], {
                dataValues: {
                    image_id: 3
                }
            });
        });

        it('should push videos onto videos list', () => {
            const relations = [
                {
                    content_item: {
                        dataValues: {
                            video_id: 1
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            video_id: 2
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            video_id: 3
                        }
                    }
                }
            ];

            const groupedRelations = groupByArticleRelations(relations);
            assert.deepEqual(groupedRelations.video[0], {
                dataValues: {
                    video_id: 1
                }
            });
            assert.deepEqual(groupedRelations.video[1], {
                dataValues: {
                    video_id: 2
                }
            });
            assert.deepEqual(groupedRelations.video[2], {
                dataValues: {
                    video_id: 3
                }
            });
        });

        it('should push galleries onto gallery list', () => {
            const relations = [
                {
                    content_item: {
                        dataValues: {
                            gallery_id: 1
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            gallery_id: 2
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            gallery_id: 3
                        }
                    }
                }
            ];

            const groupedRelations = groupByArticleRelations(relations);
            assert.deepEqual(groupedRelations.gallery[0], {
                dataValues: {
                    gallery_id: 1
                }
            });
            assert.deepEqual(groupedRelations.gallery[1], {
                dataValues: {
                    gallery_id: 2
                }
            });
            assert.deepEqual(groupedRelations.gallery[2], {
                dataValues: {
                    gallery_id: 3
                }
            });
        });

        it('should push html onto html list', () => {
            const relations = [
                {
                    content_item: {
                        dataValues: {
                            html_id: 1
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            html_id: 2
                        }
                    }
                },
                {
                    content_item: {
                        dataValues: {
                            html_id: 3
                        }
                    }
                }
            ];

            const groupedRelations = groupByArticleRelations(relations);
            assert.deepEqual(groupedRelations.html[0], {
                dataValues: {
                    html_id: 1
                }
            });
            assert.deepEqual(groupedRelations.html[1], {
                dataValues: {
                    html_id: 2
                }
            });
            assert.deepEqual(groupedRelations.html[2], {
                dataValues: {
                    html_id: 3
                }
            });
        });

        it('should ignore elements without recognized id key and null elements', () => {
            const relations = [
                {
                    dataValues: {
                        poll_id: 1
                    }
                },
                {
                    dataValues: {
                        poll_id: 2
                    }
                },
                null
            ];

            const groupedRelations = groupByArticleRelations(relations);
            assert.deepEqual(groupedRelations, {
                image: [],
                video: [],
                gallery: [],
                html: []
            });
        });

    });

});

