'use strict';

const assert = require('chai').assert;
const rewire = require('rewire');
const sinon = require('sinon');
const index = rewire('../..');
const BeneLogger = require('bene-logger');

describe('index', function () {

    const configStub = {
        server: {
            host: "localhost",
            port: 8080
        },
        db: {
            username: "blah",
            password: "blah",
            database: "blah",
            dialect: "postgres",
            host: "localhost",
            logging: false
        },
        cache: {
            enabled: false,
            redis: {
                host: 'myRedis',
                port:  '6379'
            },
            duration: '5 minutes'
        },
        logging: {
            enabled: false,
            enhanced: false
        },
        "authorsEndpoint": "http://authors-api-bertha.tm-dev-awx.com"
    };


    describe('main', () => {
        it('should pass config and logger to the database', () => {

            const Db = index.__get__('Db');
            const spy = sinon.spy(Db);
            index.__set__('Db', spy);

            index(configStub);
            const args = spy.getCall(0).args;
            assert.equal(args[0], configStub.db);
            assert.instanceOf(args[1], BeneLogger);

            index.__set__('Db', Db);
        });
    });

    describe('enhancedHealthReporting', () => {

        it('should create enhanced logging middleware', () => {

            const morganStub = sinon.stub().returns(42);
            morganStub.token = sinon.stub().returns(void 0);
            const fileStreamStub = function(filename, pattern, options) {
                return {filename, pattern, options};
            };

            const enhancedHealthReporting = index.__get__('enhancedHealthReporting');
            const result = enhancedHealthReporting(morganStub, fileStreamStub);

            assert.strictEqual(result, 42);

            const args = morganStub.getCall(0).args;
            assert.strictEqual(args[0], ':unixtime":url":status":response-time":response-time":remote-addr');
            assert.deepEqual(args[1].stream, {
                filename: '/var/log/nginx/healthd/application.log.',
                pattern: 'yyyy-MM-dd-hh',
                options: {alwaysIncludePattern: true}
            });
        });
    });

    describe('redisCache', () => {

        it('should create redisCache middleware', () => {

            const clientStub = sinon.stub().returns({redis: 'client'});
            const redis = {createClient: clientStub};

            const middlewareStub = sinon.stub().returns({func: 'req,res,next'});
            const optionsStub = sinon.stub().returns({middleware: middlewareStub});

            const apicache = {options: optionsStub};

            const redisCache = index.__get__('redisCache');
            const result = redisCache(configStub, redis, apicache);

            assert.deepEqual(result, {func: 'req,res,next'});

            sinon.assert.calledWith(clientStub, {
                host: 'myRedis',
                port:  '6379'
            });

            sinon.assert.calledWith(optionsStub, {
                redisClient: {redis: 'client'},
                statusCodes: {
                    exclude: [],
                    include: [200]
                }
            });

            sinon.assert.calledWith(middlewareStub, '5 minutes');

        });
    });
});