'use strict';

const R = require('ramda');
const sinon = require('sinon');
const assert = require('chai').assert;
const faker = require('../../faker');
const tz = require('timezone/loaded');
const rewire = require('rewire');

const config = {
    "widgetIds": {
        "online": 4705627, "offline": 5478357
    },
    "imageDimensions": {
        "width": 510,
        "height": 339,
        "code": "s510"
    }
};

const createArticle = require('../../../toJSON/article')(config);

const fakeSection = R.assoc('publication', faker.publication2, faker.section71);
const fakeArticle = R.assoc('home_section', fakeSection, faker.article);

describe('Article JSON', function () {

    const meta = {
        "type": "article"
    };

    it('should build a JSON object with the correct fields when all fields are supplied', function (done) {

        const json = createArticle([meta, fakeArticle]);
        const testDate = tz(faker.article.created_date, '%Y-%m-%dT%H:%M:%S.%3N%^z', 'UTC');
        const actual = json[1];

        assert.strictEqual(actual.data.type, 'news');
        assert.equal(actual.data.id, 123456789);
        assert.equal(actual.data.attributes.createdDate, testDate);
        assert.equal(actual.data.attributes.publishedDate, testDate);
        assert.deepEqual(actual.data.attributes.lastModifiedDate, faker.article.last_modified_date);
        assert.deepEqual(actual.data.attributes.homeSection, {name: 'new section'});
        assert.equal(actual.data.attributes.primaryTag, 'Queen');
        assert.equal(actual.data.attributes.state, 'published');
        assert.equal(actual.data.attributes.heading, 'Test Article');
        assert.equal(actual.data.attributes.advertorial, false);
        assert.equal(actual.data.attributes.sponsored, true);
        assert.equal(actual.data.attributes.subtype, 'sponsored');
        assert.equal(actual.data.attributes.exclude, false);
        assert.equal(actual.data.attributes.socialHeadline, 'Social headline');
        assert.equal(actual.data.attributes.leadText, 'This is some lead text');
        assert.equal(actual.data.attributes.leadMedia, null);
        assert.deepEqual(actual.data.attributes.tags, []);
        assert.equal(actual.data.links.ownerUrl, 'http://www.mirror.co.uk/blah/blah');
        assert.equal(actual.data.links.url, 'http://www.mirror.co.uk/incoming/file.ece?widgetId=4705627');
        assert.equal(actual.data.links.offlineUrl, 'http://www.mirror.co.uk/incoming/file.ece?widgetId=5478357&service=tablet');
        assert.deepEqual(actual.included, []);
        done();
    });

    it('should set subtype to advertorial when advertorial', function (done) {
        const json = createArticle([meta, R.assoc('sub_type', 'advertorial', fakeArticle)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.subtype, 'advertorial');
        done();
    });

    it('should set subtype to sponsored when sponsored', function (done) {
        const json = createArticle([meta, R.assoc('sub_type', 'sponsored', fakeArticle)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.subtype, 'sponsored');
        done();
    });

    it('should set subtype to null when subtype is unsupported', function (done) {
        const json = createArticle([meta, R.assoc('sub_type', 'news', fakeArticle)]);
        const actual = json[1];

        assert.isNull(actual.data.attributes.subtype);
        done();
    });

    it('should set subtype to null when subtype is undefined', function (done) {
        const json = createArticle([meta, R.assoc('sub_type', undefined, fakeArticle)]);
        const actual = json[1];

        assert.isNull(actual.data.attributes.subtype);
        done();
    });

    it('should set comments if field exists', function (done) {
        const json = createArticle([meta, R.assoc('comments', false, fakeArticle)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should set comments to false if field does not exist', function (done) {
        const json = createArticle([meta, fakeArticle]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should set comments to false if field is null', function (done) {
        const json = createArticle([meta, R.assoc('comments', null, fakeArticle)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should filter out null or undefined articleContent', () => {
        const article = R.assoc('article_contents', [1,2,3,4], fakeArticle);
        const rewiredCreateArticle = rewire('../../../toJSON/article');
        const stubbedCreateArticleContent = sinon.stub();
        stubbedCreateArticleContent.onCall(0).returns(Array.of({}, faker.articleContent[0]));
        stubbedCreateArticleContent.onCall(1).returns(null);
        stubbedCreateArticleContent.onCall(2).returns(Array.of({}, faker.articleContent[0]));
        stubbedCreateArticleContent.onCall(3).returns(void 0);
        rewiredCreateArticle.__set__('createArticleContent', stubbedCreateArticleContent);
        const json = rewiredCreateArticle(config, [meta, article]);
        assert.equal(json[1].included.length, 2);
    });

    it("should build json with webview = true for quiz content", () => {
        const article = R.assoc('article_contents', [1], fakeArticle);
        const rewiredCreateArticle = rewire('../../../toJSON/article');
        const stubbedCreateArticleContent = sinon.stub();
        stubbedCreateArticleContent.onCall(0).returns(Array.of({
            "webview": true
        }, faker.articleContent[0]));
        rewiredCreateArticle.__set__('createArticleContent', stubbedCreateArticleContent);
        const json = rewiredCreateArticle(config, [meta, article]);
        assert.isTrue(json[1].data.attributes.webview);
    });

    it('has should filter out null or undefined articleRelations', () => {
        const article = R.assoc('article_relations', [1,2,3,4], fakeArticle);
        const rewiredCreateArticle = rewire('../../../toJSON/article');
        const createLeadMedia = sinon.stub().returns(void 0);
        rewiredCreateArticle.__set__('createLeadMedia', createLeadMedia);
        const actual = rewiredCreateArticle(config, [meta, article]);
        assert.equal(actual[1].data.attributes.leadMedia, void 0);
    });

    it('should fail if no heading', () => {
        try {
            createArticle([meta, R.dissoc('heading', faker.article)]);
        } catch(e) {
            assert.equal(e.message, 'No heading found in article nationals.123456789');
            assert.equal(e.status, 'BadRequestError');
        }
    });

    it('Should fail if no section', () => {
        try {
            createArticle([meta, faker.article]);
        } catch(e) {
            assert.equal(e.message, 'Section nationals.71 not available');
            assert.equal(e.status, 'NotFoundError');
        }
    });

    it('should return undefined in toJSON method for ownerUrl if stored as null', function (done) {
        const article = R.assoc('owner_url', null, fakeArticle);
        const json = createArticle([meta, article]);
        assert.isUndefined(json[1].data.links.ownerUrl);
        done();
    });

    it('should return undefined in toJSON method for authorImageUrl if stored as null', function (done) {
        const article = R.assoc('authorImageUrl', null, fakeArticle);
        const json = createArticle([meta, article]);
        assert.isUndefined(json[1].data.links.authorImageUrl);
        done();
    });
    
    it('should add a placeholder for a leadMedia image', () => {
        const relation = faker.picturerelArticleRelation;
        const article = R.assoc('article_relations', [relation], fakeArticle);
        const actual = createArticle([meta, article])[1];
        const actualLeadMedia = actual.data.attributes.leadMedia;
        assert.equal(actualLeadMedia.image_id, 'nationals.6662380');
        assert.equal(actualLeadMedia.static_content.caption, 'an article specific caption');
        assert.equal(actualLeadMedia.noContent, true);
    });

});
