'use strict';

const R = require('ramda');
const assert = require('chai').assert;
const faker = require('../../faker');
const rewire = require('rewire');
const config = require('config');
const imageService = config.imageService;

const createImage = rewire('../../../toJSON/image');

describe('Image JSON', function () {

    const imageSize = {
        width: 320,
        height: 150,
        code: '1234b'
    };

    const fakeSection = faker.section71;
    const testImage = R.assoc('home_section', fakeSection, faker.image);

    it('should build a meta data object', function () {
        const actual = createImage(imageSize, [{}, testImage]);
        const meta = actual[0];
        assert.equal(meta.type, 'image');
        assert.equal(meta.lastModifiedDate, testImage.last_modified_date);
        assert.equal(meta.id, testImage.id);
    });

    it('should build a JSON object with the correct fields when a publication is included', function () {
        const actual = createImage(imageSize, [{}, testImage]);
        const json = actual[1];
        assert.equal(json.data.type, 'image');
        assert.equal(json.data.id, 6662380);
        assert.equal(json.data.attributes.imageCredit, 'test image credit');
        assert.equal(json.data.attributes.width, 320);
        assert.equal(json.data.attributes.height, 150);
        assert.equal(json.data.attributes.caption, 'test caption');
        assert.equal(json.data.attributes.altText, 'test alttext');
        assert.equal(json.data.attributes.title, 'test-title');
        assert.equal(json.data.attributes.alternates.length, 6);
        assert.equal(json.data.attributes.alternates[0].crop, 's510');
        assert.equal(json.data.attributes.alternates[0].height, 339);
        assert.equal(json.data.attributes.alternates[0].width, 510);
        assert.equal(json.data.links.url, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/1234b/test-title`);
        assert.equal(json.data.links.thumbnailUrl, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s338b/test-title`);
        assert.equal(json.data.links.templateUrl, `https://${imageService}.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/%s/test-title`);
    });

    it('createImageUrl() should create a valid URL for www.mirror.co.uk', () => {
        const createImageUrl = createImage.__get__('createImageUrl');
        const actual = createImageUrl(faker.image, {url: 'http://www.mirror.co.uk/news/test/'}, '1234b');
        assert.equal(actual, `https://${imageService}.mirror.co.uk/news/test/article6662380.ece/ALTERNATES/1234b/test-title`);
    });

    it('createImageUrl() should create a valid URL for www2.ok.co.uk', () => {
        const createImageUrl = createImage.__get__('createImageUrl');
        const actual = createImageUrl(faker.image, {url: 'http://www2.ok.co.uk/news/test/'}, '1234b');
        assert.equal(actual, `https://${imageService}.ok.co.uk/news/test/article6662380.ece/ALTERNATES/1234b/test-title`);
    });

    it('createImageUrl() should create a valid URL for www3.dailystar.co.uk', () => {
        const createImageUrl = createImage.__get__('createImageUrl');
        const actual = createImageUrl(faker.image, {url: 'http://www3.dailystar.co.uk/news/test/'}, '1234b');
        assert.equal(actual, `https://${imageService}.dailystar.co.uk/news/test/article6662380.ece/ALTERNATES/1234b/test-title`);
    });

    it('buildAlternates() should build an alternates', () => {
        const buildAlternates = createImage.__get__('buildAlternates');
        const actual = buildAlternates([
            "s510",
            "s482b",
            "s510b",
            "s170",
            "s338b",
            "s260",
            "s615",
            "s615b",
            "s458",
            "s308"
        ], faker.image.alternates);
        assert.equal(actual.length, 5);
        assert.equal(actual[0].crop, 's510');
        assert.equal(actual[0].height, 339);
        assert.equal(actual[0].width, 510);
        assert.equal(actual[0].width, 510);
        assert.isUndefined(R.find(R.propEq('crop', 's1000000000b'), actual));
    });

    it('buildAlternates() should return null when alternates is null', () => {
        const buildAlternates = createImage.__get__('buildAlternates');
        const actual = buildAlternates([
            "s510",
            "s482b",
            "s510b",
            "s170",
            "s338b",
            "s260",
            "s615",
            "s615b",
            "s458",
            "s308"
        ], faker.image2.alternates);
        assert.isNull(actual);
    });

    it('buildAlternates() should return null when alternates list is an empty list', () => {
        const buildAlternates = createImage.__get__('buildAlternates');
        const actual = buildAlternates(["s9999"], faker.image.alternates);
        assert.isNull(actual);
    });

    it('creates an image with an article relation', () => {
        const testImageWithRelation = R.assoc('article_relations', [faker.imageArticleRelation], testImage);
        const actual = createImage(imageSize, [{}, testImageWithRelation]);
        assert.equal(actual[1].data.attributes.caption, 'an article specific caption');
        assert.equal(actual[1].data.attributes.altText, 'some updated alttext');
    });

    it("returns null if the images doesn't have a home_section", () => {
        const noSectionImage = R.assoc('home_section', void 0, faker.image);
        const actual = createImage(imageSize, [{}, noSectionImage]);
        assert.isNull(actual);
    });

});
