'use strict';

const sinon = require('sinon');
const assert = require('chai').assert;
const rewire = require('rewire');

const imageSizeFromConfig = {
    width: 300,
    height: 150,
    code: 's1234b'
};

describe('Lead Media JSON', function () {

    describe('leadMedia', () => {
        const rewiredCreateLeadMedia = rewire('../../../toJSON/leadMedia');

        const createArticleContentStub = sinon.stub().returns([{}, { message: 'creates an article content' }]);

        rewiredCreateLeadMedia.__set__('createArticleContent', createArticleContentStub);

        afterEach(() => createArticleContentStub.reset());

        it('should handle an image led article', () => {
            const relations = [
                {
                    relation_type: 'picturerel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                }
            ];

            assert.equal(rewiredCreateLeadMedia(imageSizeFromConfig, relations)[1].message, 'creates an article content');
            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'picturerel', order_index: 0, content_item: {image_id: 'nationals.12345'}, static_content: {foo: 'bar'}}
            );
        });

        it('should handle a video led article with a picturerel', () => {
            const relations = [
                {
                    relation_type: 'picturerel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                },
                {
                    relation_type: 'videorel',
                    order_index: 0,
                    content_item: {
                        video_id: 'nationals.67891'
                    },
                    static_content: {
                        baz: 'qux'
                    }
                }
            ];
            assert.equal(rewiredCreateLeadMedia(imageSizeFromConfig, relations)[1].message, 'creates an article content');

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'picturerel', order_index: 0, content_item: {image_id: 'nationals.12345'}, static_content: {foo: 'bar'}}
            );

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'videorel', order_index: 0, content_item: {video_id: 'nationals.67891'}, static_content: {baz: 'qux'}}
            );
        });

        it('should handle a video led article with a teaserrel', () => {
            const relations = [
                {
                    relation_type: 'teaserrel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                },
                {
                    relation_type: 'videorel',
                    order_index: 0,
                    content_item: {
                        video_id: 'nationals.67891'
                    },
                    static_content: {
                        baz: 'qux'
                    }
                }
            ];
            assert.equal(rewiredCreateLeadMedia(imageSizeFromConfig, relations)[1].message, 'creates an article content');

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'teaserrel', order_index: 0, content_item: {image_id: 'nationals.12345'}, static_content: {foo: 'bar'}}
            );

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'videorel', order_index: 0, content_item: {video_id: 'nationals.67891'}, static_content: {baz: 'qux'}}
            );
        });

        it('should handle a video led article with a teaserrel and a picturerel', () => {
            const relations = [
                {
                    relation_type: 'picturerel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                },
                {
                    relation_type: 'teaserrel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                },
                {
                    relation_type: 'videorel',
                    order_index: 0,
                    content_item: {
                        video_id: 'nationals.67891'
                    },
                    static_content: {
                        baz: 'qux'
                    }
                }
            ];
            assert.equal(rewiredCreateLeadMedia(imageSizeFromConfig, relations)[1].message, 'creates an article content');

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'picturerel', order_index: 0, content_item: {image_id: 'nationals.12345'}, static_content: {foo: 'bar'}}
            );

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'videorel', order_index: 0, content_item: {video_id: 'nationals.67891'}, static_content: {baz: 'qux'}}
            );
        });

        it('should handle a gallery led article', () => {
            const relations = [
                {
                    relation_type: 'picturerel',
                    order_index: 0,
                    content_item: {
                        image_id: 'nationals.12345'
                    },
                    static_content: {
                        foo: 'bar'
                    }
                },
                {
                    relation_type: 'videorel',
                    order_index: 0,
                    content_item: {
                        video_id: 'nationals.67891'
                    },
                    static_content: {
                        baz: 'qux'
                    }
                },
                {
                    relation_type: 'picturerel',
                    order_index: 0,
                    content_item: {
                        gallery_id: 'nationals.235711'
                    },
                    static_content: {
                        quux: 'corge'
                    }
                }
            ];

            assert.equal(rewiredCreateLeadMedia(imageSizeFromConfig, relations)[1].message, 'creates an article content');

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'picturerel', order_index: 0, content_item: {gallery_id: 'nationals.235711'}, static_content: {quux: 'corge'}}
            );

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'picturerel', order_index: 0, content_item: {image_id: 'nationals.12345'}, static_content: {foo: 'bar'}}
            );

            sinon.assert.calledWith(createArticleContentStub,
                {width: 300, height: 150, code: 's1234b'},
                {relation_type: 'videorel', order_index: 0, content_item: {video_id: 'nationals.67891'}, static_content: {baz: 'qux'}}
            );
        });

    });

});
