'use strict';

const R = require('ramda');
const assert = require('chai').assert;
const faker = require('../../faker');
const tz = require('timezone/loaded');

const config = {
    "widgetIds": {
        "online": 4705627,"offline": 5478357
    },
    "imageDimensions": {
        "width": 510,
        "height": 339,
        "code": "s510"
    }
};

const createLiveEvent = require('../../../toJSON/article')(config);

const fakeSection = R.assoc('publication', faker.publication2, faker.section71);
const fakeLiveEvent = R.assoc('home_section', fakeSection, faker.liveEvent);

describe('Live Event JSON', function () {

    const meta = {
        type: 'live-event'
    };

    it('should build a JSON object with the correct fields when all fields are supplied', () => {
        const tuple = createLiveEvent([meta, fakeLiveEvent]);
        const testDate = tz(faker.liveEvent.created_date, '%Y-%m-%dT%H:%M:%S.%3N%^z', 'UTC');
        const actual = R.last(tuple);

        assert.strictEqual(actual.data.type, 'live-event');
        assert.strictEqual(actual.data.id, 9014003);
        assert.strictEqual(actual.data.attributes.createdDate, testDate);
        assert.strictEqual(actual.data.attributes.publishedDate, testDate);
        assert.strictEqual(actual.data.attributes.lastModifiedDate, faker.liveEvent.last_modified_date);
        assert.deepEqual(actual.data.attributes.homeSection, { name: 'new section' });
        assert.strictEqual(actual.data.attributes.primaryTag, null);
        assert.strictEqual(actual.data.attributes.state, 'published');
        assert.strictEqual(actual.data.attributes.sponsored, false);
        assert.strictEqual(actual.data.attributes.title, 'Fake Live Event');
        assert.strictEqual(actual.data.attributes.leadMedia, null);
        assert.isTrue(actual.data.attributes.webview);
        assert.strictEqual(actual.data.links.ownerUrl, 'http://www.mirror.co.uk/live-event-9014003');
        assert.strictEqual(actual.data.links.url, 'http://www.mirror.co.uk/incoming/filename.ece?widgetId=4705627');
        assert.strictEqual(actual.data.links.offlineUrl, 'http://www.mirror.co.uk/incoming/filename.ece?widgetId=5478357&service=tablet');
        assert.deepEqual(actual.included, []);
    });

    it('should set comments if field exists', function (done) {
        const json = createLiveEvent([meta, R.assoc('comments', false, fakeLiveEvent)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should set comments to false if field does not exist', function (done) {
        const json = createLiveEvent([meta, fakeLiveEvent]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should set comments to false if field is null', function (done) {
        const json = createLiveEvent([meta, R.assoc('comments', null, fakeLiveEvent)]);
        const actual = json[1];

        assert.equal(actual.data.attributes.comments, false);
        done();
    });

    it('should not have an owner_url field in the JSON if absent in the query', () => {
        const article = R.assoc('owner_url', null, fakeLiveEvent);
        const json = R.last(createLiveEvent([meta, article]));
        assert.isUndefined(json.data.links.ownerUrl);
    });
});
