'use strict';

const assert = require('chai').assert;
const R = require('ramda');
const rewire = require('rewire');
const config = require("config");
const stitcher = rewire('../../../toJSON/stitcher');
const createIncludedList = stitcher.__get__('createIncludedList');
const isWebviewArticle = stitcher.__get__('isWebviewArticle');
const cropLeadImages = stitcher.__get__('cropLeadImages');
const formatGalleries = stitcher.__get__('formatGalleries');
const filterAlternates = stitcher.__get__('filterAlternates');
const deduplicateAuthors = stitcher.__get__('deduplicateAuthors');

describe('JSON Stitcher', () => {

    const articleMeta = {
        type: 'article'
    };

    const article = {
        type: 'article',
        data: {
            attributes: {
                leadMedia: null
            }
        },
        included: [
            {
                noContent: true,
                id: 'nationals.1234'
            },
            {
                noContent: true,
                id: 'notexisting.404'
            }
        ]
    };

    const articleTuple = [articleMeta, article];

    const gallery = [{
        "type": "gallery",
        "id": 'nationals.1234'
    }, {
        data: {
            type: 'gallery',
            id: 1234,
            attributes: {}
        }
    }];

    it('sets social headline to null if empty', () => {
        const given = [[articleMeta, R.assocPath(['data', 'attributes', 'socialHeadline'], '', article)]];
        const actual = stitcher(given);
        assert.isNull(actual.data.attributes.socialHeadline);
    });

    it('removes ownerUrl if it is an empty string', () => {
        const given = [[articleMeta, R.assocPath(['data', 'links', 'ownerUrl'], '', article)]];
        const actual = stitcher(given);
        assert.isUndefined(actual.data.links.ownerUrl);
    });

    it('puts a content item into the included array of an article', () => {
        const given = [articleTuple, gallery];
        const actual = stitcher(given);
        assert.deepEqual(actual.included[0], given[1][1]);

        // the second item should be filtered out.
        assert.equal(actual.included.length, 1);
    });

    it('puts a content item into the included array of an article with merged static content', () => {
        const given = [
            [{}, {
                data: {
                    attributes: {
                        leadMedia: null
                    }
                },
                included: [
                    {
                        noContent: true,
                        id: 'nationals.1234',
                        static_content: {
                            caption: 'just your bog standard, run of the mill caption'
                        }
                    }
                ]
            }], gallery];
        const actual = stitcher(given).included[0];
        assert.equal(actual.data.attributes.caption, 'just your bog standard, run of the mill caption');
    });

    it('adds an array of authors to an article', () => {

        const authorMeta = {
            "type": "author"
        };

        const given = [
            [articleMeta, {
                data: {
                    attributes: {
                        leadMedia: null
                    }
                },
                included: []
            }],
            [authorMeta, {

                type: 'author',
                name: 'Leonardo'
            }],
            [authorMeta, {
                type: 'author',
                name: 'Michaelangelo'
            }],
            [authorMeta, {
                type: 'author',
                name: 'Raphael'
            }],
            [authorMeta, {
                type: 'author',
                name: 'Donatello'
            }]
        ];
        const actual = stitcher(given);
        assert.equal(actual.data.attributes.authors[0].name, 'Leonardo');
        assert.equal(actual.data.attributes.authors[1].name, 'Michaelangelo');
        assert.equal(actual.data.attributes.authors[2].name, 'Raphael');
        assert.equal(actual.data.attributes.authors[3].name, 'Donatello');
    });

    it('sets a content item as the lead media if lead media has noContent set to true', () => {
        const given = [
            [articleMeta, {
                data: {
                    attributes: {
                        leadMedia: {
                            id: 'nationals.1234',
                            noContent: true
                        }
                    }
                },
                included: []
            }], gallery];
        const leadMedia = stitcher(given).data.attributes.leadMedia;
        assert.equal(leadMedia.data.id, 1234);
        assert.equal(leadMedia.data.type, 'gallery');
    });

    it('sets lead media to null if no matching content is found', () => {
        const given = [
            [articleMeta, {
                data: {
                    attributes: {
                        leadMedia: {
                            id: 'nationals.1235',
                            noContent: true
                        }
                    }
                },
                included: []
            }], gallery];

        const leadMedia = stitcher(given).data.attributes.leadMedia;
        assert.equal(leadMedia, null);
    });

    it('does not alter lead media if noContent is undefined', () => {
        const given = [
            [articleMeta, {
                data: {
                    attributes: {
                        leadMedia: {
                            data: {
                                id: 'nationals.1234',
                                type: 'image'
                            }
                        }
                    }
                },
                included: []
            }]
        ];
        const leadMedia = stitcher(given).data.attributes.leadMedia;
        assert.equal(leadMedia.data.id, 'nationals.1234');
        assert.equal(leadMedia.data.type, 'image');
    });

});

describe('createIncludedList()', () => {

    it('should return an article with an included list of paragraphs unchanged', done => {
        const article = {
            included: [
                {
                    data: {
                        type: 'text',
                        attributes: {
                            body: 'blah blah'
                        }
                    }
                },
                {
                    data: {
                        type: 'text',
                        attributes: {
                            body: 'more text'
                        }
                    }
                }
            ]
        };
        assert.deepEqual(createIncludedList(article, []), article.included);
        done();
    });

    it('should get unresolved included elements from tail list', done => {
        const tail = [
            [
                {
                    "type": "image",
                    "lastModifiedDate": "2016-10-25T10:36:35.247Z",
                    "id": "nationals.6662380"
                },
                {
                    "data": {
                        "type": "image",
                        "id": 6662380,
                        "attributes": {
                            "imageCredit": "test image credit",
                            "width": 510,
                            "height": 339,
                            "altText": "test alttext",
                            "caption": "test caption",
                            "title": "test-title"
                        },
                        "links": {
                            "url": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510/test-title",
                            "thumbnailUrl": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s338b/test-title"
                        }
                    }
                }
            ]
        ];
        const article = {
            included: [
                {
                    data: {
                        type: 'text',
                        attributes: {
                            body: 'blah blah'
                        }
                    }
                },
                {
                    "noContent": true,
                    "id": "nationals.6662380",
                    "static_content": {
                        "caption": "just your bog standard, run of the mill caption"
                    }
                }

            ]
        };
        const actual = createIncludedList(article, tail);
        assert.equal(actual.length, 2);
        assert.equal(R.path(['data', 'type'], actual[1]), 'image');
        assert.equal(R.path(['data', 'attributes', 'caption'], actual[1]), 'just your bog standard, run of the mill caption');
        done();
    });

    it('should remove content items from included list when no matching id is found', done => {
        const tail = [
            [
                {
                    "type": "image",
                    "lastModifiedDate": "2016-10-25T10:36:35.247Z",
                    "id": "nationals.12345"
                },
                {
                    "data": {
                        "type": "image",
                        "id": 12345,
                        "attributes": {
                            "imageCredit": "test image credit",
                            "width": 510,
                            "height": 339,
                            "altText": "test alttext",
                            "caption": "test caption",
                            "title": "test-title"
                        },
                        "links": {
                            "url": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510/test-title",
                            "thumbnailUrl": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s338b/test-title"
                        }
                    }
                }
            ]
        ];
        const article = {
            included: [
                {
                    data: {
                        type: 'text',
                        attributes: {
                            body: 'blah blah'
                        }
                    }
                },
                {
                    "noContent": true,
                    "id": "nationals.6662380",
                    "static_content": {
                        "caption": "just your bog standard, run of the mill caption"
                    }
                }

            ]
        };
        const actual = createIncludedList(article, tail);
        assert.equal(actual.length, 1);
        done();
    });

});

describe('isWebviewArticle()', () => {

    it('should return unchanged article if webview is already set to true', done => {
        const article = {
            data: {
                attributes: {
                    webview: true
                }
            }
        };
        assert.equal(isWebviewArticle(article, []), true);
        done();
    });

    it('should flip webview to true if article contains a quiz', done => {
        const article = {
            data: {
                attributes: {
                    webview: false
                }
            }
        };
        const included= [
            {
                data:{
                    type: 'text'
                }
            },
            {
                data:{
                    type: 'quiz'
                }
            },
            {
                data:{
                    type: 'image'
                }
            }
        ];
        assert.equal(isWebviewArticle(article, included), true);
        done();
    });

    it('should flip webview to true if article contains an interactiveHtml', done => {
        const article = {
            data: {
                attributes: {
                    webview: false
                }
            }
        };
        const included= [
            {
                data:{
                    type: 'text'
                }
            },
            {
                data:{
                    type: 'interactiveHtml'
                }
            },
            {
                data:{
                    type: 'image'
                }
            }
        ];
        assert.equal(isWebviewArticle(article, included), true);
        done();
    });

    it('should not flip webview to true if all included elements are supported natively', done => {
        const article = {
            data: {
                attributes: {
                    webview: false
                }
            }
        };
        const included= [
            {
                data:{
                    type: 'text'
                }
            },
            {
                data:{
                    type: 'image'
                }
            }
        ];
        assert.equal(isWebviewArticle(article, included), false);
        done();
    });

});

describe('cropLeadImages()', () => {

    it('should crop a lead image', () => {
        const leadImage = {
            data: {
                type: 'image',
                attributes: {
                    width: 510,
                    height: 382,
                    code: 's510b'
                },
                links: {
                    url: 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510b/test-title',
                    thumbnailUrl: 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510b/test-title'
                }
            }
        };

        const result = cropLeadImages(config, leadImage);

        assert.equal(result.data.attributes.width, 510);
        assert.equal(result.data.attributes.height, 339);
        assert.equal(result.data.links.url, 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510/test-title');
        assert.equal(result.data.links.thumbnailUrl, 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s170/test-title');
    });

    it('should crop a lead image in a lead video', () => {
        const leadVideo = {
            data: {
                type: 'video',
                attributes: {
                    leadMedia: {
                        data: {
                            type: 'image',
                            attributes: {
                                width: 510,
                                height: 382,
                                code: 's510b'
                            },
                            links: {
                                url: 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510b/test-title'
                            }
                        }
                    }
                }
            }
        };

        const result = cropLeadImages(config, leadVideo);
        const image = result.data.attributes.leadMedia;

        assert.equal(image.data.attributes.width, 510);
        assert.equal(image.data.attributes.height, 339);
        assert.equal(image.data.attributes.code, 's510');
        assert.equal(image.data.links.url, 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510/test-title');
    });

    it('should crop a lead image in a lead gallery', () => {
        const leadVideo = {
            data: {
                type: 'gallery',
                attributes: {
                    leadMedia: {
                        data: {
                            type: 'image',
                            attributes: {
                                width: 510,
                                height: 382,
                                code: 's510b'
                            },
                            links: {
                                url: 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510b/test-title'
                            }
                        }
                    }
                }
            }
        };

        const result = cropLeadImages(config, leadVideo);
        const image = result.data.attributes.leadMedia;

        assert.equal(image.data.attributes.width, 510);
        assert.equal(image.data.attributes.height, 339);
        assert.equal(image.data.attributes.code, 's510');
        assert.equal(image.data.links.url, 'http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s510/test-title');
    });

});

describe('formatGalleries()', () => {

    const tail = [[{
        "type": "gallery",
        "id": 'nationals.1234'
    }, {
        data: {
            type: 'gallery',
            id: 1234,
            attributes: {
                leadMedia: {
                    "data": {
                        "type": "image",
                        "id": 6662380,
                        "attributes": {
                            "imageCredit": "test image credit",
                            "width": 10,
                            "height": 10,
                            "altText": "test alttext",
                            "caption": "test caption",
                            "title": "test-title"
                        },
                        "links": {
                            "url": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s100/test-title",
                            "thumbnailUrl": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s338b/test-title"
                        }
                    }
                }
            }
        },
        included: [
            {
                "data": {
                    "type": "image",
                    "id": 6662380,
                    "attributes": {
                        "imageCredit": "test image credit",
                        "width": 10,
                        "height": 10,
                        "altText": "test alttext",
                        "caption": "test caption",
                        "title": "test-title"
                    },
                    "links": {
                        "url": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s100/test-title",
                        "thumbnailUrl": "http://www.mirror.co.uk/incoming/article6662380.ece/ALTERNATES/s338b/test-title"
                    }
                }
            }
        ]
    }]];

    it('should format gallery images', () => {
        const gallery = R.head(formatGalleries(config, tail))[1];
        const leadMedia = gallery.data.attributes.leadMedia;
        const includedImage = gallery.included[0];

        assert.equal(leadMedia.data.attributes.width, 510);
        assert.equal(leadMedia.data.attributes.height, 339);
        assert.equal(leadMedia.data.attributes.code, 's510');
        assert.equal(includedImage.data.attributes.width, 1227);
        assert.equal(includedImage.data.attributes.height, 1000);
        assert.equal(includedImage.data.attributes.code, 's1227b');
    });

});

describe('filterAlternates()', () => {
    it('should return null if alternates are null', () => {
        const actual = filterAlternates('anything', null);
        assert.isNull(actual);
    });

    it('should return null if filtered list is empty', () => {
        const actual = filterAlternates('leadMedia', [{crop: 's170b'}, {crop: 's270b'}]);
        assert.isNull(actual);
    });

    it('should return list without b crops if context is leadMedia', () => {
        const actual = filterAlternates('leadMedia', [{crop: 's170b'}, {crop: 's170'}, {crop: 's270'}, {crop: 's270b'}]);
        assert.equal(actual.length, 2);
        assert.equal(actual[0].crop, 's170');
        assert.equal(actual[1].crop, 's270');
    });

    it('should return list with only s1227b crop if context is gallery', () => {
        const actual = filterAlternates('gallery', [{crop: 's170b'}, {crop: 's170'}, {crop: 's1227b'}]);
        assert.equal(actual.length, 1);
        assert.equal(actual[0].crop, 's1227b');
    });
});

describe('deduplicateAuthors()', () => {

    it('should return a list of non duplicated authors unchanged', () => {
        const authors = [
            { name: 'David Bowie', id: 1, type: 'author' },
            { name: 'Lou Reed', id: 2, type: 'author' },
            { name: 'Iggy Pop', id: 3, type: 'author' }
        ];

        const actual = deduplicateAuthors(authors);
        assert.deepEqual(actual, authors);
    });

    it('should remove a duplicated author if name is duplicated and duplicate has no id', () => {
        const authors = [
            { name: 'David Bowie', id: 1, type: 'author' },
            { name: 'David Bowie', id: null, type: 'author' },
            { name: 'Lou Reed', id: 2, type: 'author' },
            { name: 'Iggy Pop', id: 3, type: 'author' }
        ];

        const actual = deduplicateAuthors(authors);
        assert.deepEqual(actual, [authors[0], authors[2], authors[3]]);
    });

    it('should not remove a duplicated author if name is duplicated but duplicate has an id', () => {
        const authors = [
            { name: 'David Bowie', id: 1, type: 'author' },
            { name: 'David Bowie', id: 4, type: 'author' },
            { name: 'Lou Reed', id: 2, type: 'author' },
            { name: 'Iggy Pop', id: 3, type: 'author' }
        ];

        const actual = deduplicateAuthors(authors);
        assert.deepEqual(actual, authors);
    });

    it('should not fail with empty list', () => {
        const authors = [];
        const actual = deduplicateAuthors(authors);
        assert.deepEqual(actual, authors);
    });

});

describe('Tag filtering process', () => {
    const resolveTags = stitcher.__get__('resolveTags');
    const removeTag = stitcher.__get__('removeTag');
    const prependTag = stitcher.__get__('prependTag');
    const evolveTags = stitcher.__get__('evolveTags');

    describe('resolveTags', () => {
        it('should, given a valid tail, produce a list of tag names', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = resolveTags(fakeTail);
            const expected = ['Murder', 'Crime'];
            assert.deepEqual(measured, expected);

        });
        it('should, given a tail that has no tags, produce an empty list', () => {
            const fakeTail = [
                [
                    { type: 'not-a-tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'not-a-tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = resolveTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an empty tail, produce an empty list', () => {
            const fakeTail = [];

            const measured = resolveTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an null tail, produce an empty list', () => {
            const fakeTail = null;

            const measured = resolveTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an undefined tail, produce an empty list', () => {
            const fakeTail = undefined;

            const measured = resolveTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
    });

    describe('removeTag', () => {
        const fakePrimaryTag = 'The Good Place';
        it('should remove the primary tag from a list of tags (Strings)', () => {
            const fakeListOfTags = ['tag 1', 'tag 2', 'The Good Place'];
            const measured = removeTag(fakePrimaryTag)(fakeListOfTags);
            const expected = ['tag 1', 'tag 2'];
            assert.deepEqual(measured, expected);
        });
        it('should not alter the list if the primary tag is not there', () => {
            const fakeListOfTags = ['tag 1', 'tag 2', 'The Good Place'];
            const measured = removeTag('not in the list')(fakeListOfTags);
            const expected = ['tag 1', 'tag 2', 'The Good Place'];
            assert.deepEqual(measured, expected);
        });
        it('should not alter the list if the tag list is empty', () => {
            const fakeListOfTags = [];
            const measured = removeTag(fakePrimaryTag)(fakeListOfTags);
            const expected = [];
            assert.deepEqual(measured, expected);
        });
        it('should return an empty list given a null input', () => {
            const fakeListOfTags = null;
            const measured = removeTag(fakePrimaryTag)(fakeListOfTags);
            const expected = [];
            assert.deepEqual(measured, expected);
        });
        it('should return an empty list given a undefined input', () => {
            const fakeListOfTags = undefined;
            const measured = removeTag(fakePrimaryTag)(fakeListOfTags);
            const expected = [];
            assert.deepEqual(measured, expected);
        });
    });

    describe('prependTag', () => {
        const fakePrimaryTag = 'The Good Place';
        it('should prepend the primary tag in a list of tags (Strings)', () => {
            const fakeListOfTags = ['tag 1', 'tag 2'];
            const measured = prependTag(fakePrimaryTag)(fakeListOfTags);
            const expected = ['The Good Place', 'tag 1', 'tag 2'];
            assert.deepEqual(measured, expected);
        });
        it('should add the primary tag to an empty tag list', () => {
            const fakeListOfTags = [];
            const measured = prependTag(fakePrimaryTag)(fakeListOfTags);
            const expected = ['The Good Place'];
            assert.deepEqual(measured, expected);
        });
        it('should return a list including the primary tag given a null input', () => {
            const fakeListOfTags = null;
            const measured = prependTag(fakePrimaryTag)(fakeListOfTags);
            const expected = ['The Good Place'];
            assert.deepEqual(measured, expected);
        });
        it('should return a list including the primary tag given a undefined input', () => {
            const fakeListOfTags = undefined;
            const measured = prependTag(fakePrimaryTag)(fakeListOfTags);
            const expected = ['The Good Place'];
            assert.deepEqual(measured, expected);
        });
    });
    describe('evolveTags', () => {
        const fakePrimaryTag = 'Primary';

        it('should, given a valid tail, return a list of tags with the primary tag at position 0', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Primary',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = evolveTags(fakePrimaryTag)(fakeTail);
            const expected = ['Primary', 'Murder', 'Crime'];

            assert.deepEqual(measured, expected);
        });
        it('should, given a valid tail, return a list of tags with the primary tag at position 0 even if the primary tag was not in the tag list', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = evolveTags(fakePrimaryTag)(fakeTail);
            const expected = ['Primary', 'Murder', 'Crime'];

            assert.deepEqual(measured, expected);
        });

        it('should, given a valid tail and a null Primary Tag, return the list of tags unchanged', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = evolveTags(null)(fakeTail);
            const expected = ['Murder', 'Crime'];

            assert.deepEqual(measured, expected);
        });
        it('should, given a valid tail and an undefined Primary Tag, return the list of tags unchanged', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = evolveTags(undefined)(fakeTail);
            const expected = ['Murder', 'Crime'];

            assert.deepEqual(measured, expected);
        });
        
        it('should, given a valid tail and an empty string Primary Tag, tolerate the empty string', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = evolveTags('')(fakeTail);
            const expected = ['', 'Murder', 'Crime'];

            assert.deepEqual(measured, expected);
        });
    });
});

describe('Search tag format process', () => {
    const resolveSearchTags = stitcher.__get__('resolveSearchTags');
    const formatSearchTags = stitcher.__get__('formatSearchTags');
    const evolveSearchTags = stitcher.__get__('evolveSearchTags');

    describe('resolveSearchTags', () => {
        it('should, given a valid tail, produce a list of tag obejcts ', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = resolveSearchTags(fakeTail);
            const expected = [{
                name: 'Murder',
                id: 'some:other:tag:id',
                tag_id: 'nationals.33'
            },
            {
                name: 'Crime',
                id: 'some:tag:id',
                tag_id: 'nationals.3'
            }];
            assert.deepEqual(measured, expected);
        });
        it('should, given a tail that has no tags, produce an empty list', () => {
            const fakeTail = [
                [
                    { type: 'not-a-tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'not-a-tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id',
                        tag_id: 'nationals.3'
                    }
                ]
            ];

            const measured = resolveSearchTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an empty tail, produce an empty list', () => {
            const fakeTail = [];

            const measured = resolveSearchTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an null tail, produce an empty list', () => {
            const fakeTail = null;

            const measured = resolveSearchTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
        it('should, given an undefined tail, produce an empty list', () => {
            const fakeTail = undefined;

            const measured = resolveSearchTags(fakeTail);
            const expected = [];
            assert.deepEqual(measured, expected);

        });
    });

    describe('formatSearchTags', () => {
        it('should return the correct format', () => {
            const tag = {
                name: 'Murder',
                id: 'some:other:tag:id',
                tag_id: 'nationals.33'
            };
            const expectedTag = {
                name: 'Murder',
                id: 'some:other:tag:id',
                tag_id: 33
            };
            const measured = formatSearchTags(tag);
            assert.deepEqual(measured, expectedTag);
        });
        it('should return the empty object for empty tag', () => {
            const tag = {};
            const expectedTag = {};
            const measured = formatSearchTags(tag);
            assert.deepEqual(measured, expectedTag);
        });
        it('should return the original tag', () => {
            const tag = {
                name: 'Murder',
                id: 'some:other:tag:id'
            };
            const expectedTag = {
                name: 'Murder',
                id: 'some:other:tag:id'
            };
            const measured = formatSearchTags(tag);
            assert.deepEqual(measured, expectedTag);
        });
    });

    describe('evolveSearchTags', () => {
        it('should, given a valid tail, return a list of tags with updated ids', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Primary',
                        id: 'some:other:tag:id',
                        tag_id: 'nationals.33'
                    }
                ]
            ];

            const measured = evolveSearchTags(fakeTail);
            const expected = [
                {
                    name: 'Murder',
                    id: 'some:other:tag:id',
                    tag_id: 33
                },
                {
                    name: 'Primary',
                    id: 'some:other:tag:id',
                    tag_id: 33
                }
            ];

            assert.deepEqual(measured, expected);
        });

        it('should, given a valid tail without tags should return the same', () => {
            const fakeTail = [
                [
                    { type: 'tag' },
                    {
                        name: 'Murder',
                        id: 'some:other:tag:id'
                    }
                ],
                [
                    { type: 'tag' },
                    {
                        name: 'Crime',
                        id: 'some:tag:id'
                    }
                ]
            ];

            const measured = evolveSearchTags(fakeTail);
            const expected = [
                {
                    name: 'Murder',
                    id: 'some:other:tag:id'
                },
                {
                    name: 'Crime',
                    id: 'some:tag:id'
                }
            ];

            assert.deepEqual(measured, expected);
        });
    });
});