'use strict';

const R = require('ramda');
const config = require('config');
let getLastUpdated = require("./getLastUpdated");
let tz = require('timezone/loaded');

const emptyOrNull = R.either(R.isNil, R.isEmpty);

const resolveTags = R.compose(
    R.pluck('name'),
    R.map(R.last),
    R.filter(R.pathEq([0, 'type'], 'tag')),
    R.defaultTo([])
);
const removeTag = tag => R.compose(R.reject(R.equals(tag)), R.defaultTo([]));
const prependTag = tag => R.compose(R.prepend(tag), R.defaultTo([]));
const evolveTags = primaryTag => R.compose(
    R.reject(R.isNil),
    prependTag(primaryTag),
    removeTag(primaryTag),
    resolveTags
);
const resolveSearchTags = R.compose(
    R.map(R.last),
    R.filter(R.pathEq([0, 'type'], 'tag')),
    R.defaultTo([])
);
const formatSearchTags = tag => {
    const tag_id = R.path(['tag_id'], tag);
    if (tag_id) {
        const id = tag_id.split(".")[1];
        return R.assoc('tag_id',parseInt(id), tag);
    } else {
        return tag;
    }
};

const evolveSearchTags =  R.compose(
        R.map(formatSearchTags),
        resolveSearchTags
);

module.exports = arrayOfObjects => {
    const articleTuple = R.head(arrayOfObjects);
    let article = articleTuple[1];
    const leadMedia = R.path(["data", "attributes", "leadMedia"], article);
    const tail = R.pipe(
        // put any tail transformations here
        formatGalleries(config)
    )(R.reject(R.isNil, R.tail(arrayOfObjects)));

    const lastModifiedDate = R.path(["data", "attributes", "lastModifiedDate"], article) != null
        ? tz(getLastUpdated(article.data.attributes.lastModifiedDate, tail), '%Y-%m-%dT%H:%M:%S.%3N%^z', 'UTC')
        : void 0;

    const ownerUrl = emptyOrNull(R.path(["data", "links", "ownerUrl"], article)) ? void 0 : article.data.links.ownerUrl;
    const socialHeadline = emptyOrNull(R.path(["data", "attributes", "socialHeadline"], article)) ? null : article.data.attributes.socialHeadline;

    // Inline images must be formated at last moment to preserve leadmedia crop behaviour
    let included = createIncludedList(article, formatInlineImages(tail));
    const urlTransform = R.curry((section, filename, id) => {
        return `${section.url}${filename}/embedded-webview/${section.publication.name}-${id}`;
    });

    // super hacky solution to solve the HTML Embed problem:
    // we needed in scope section, publication and filename of the article
    // to build up the embed url and could not resolve it from the relational model
    // Resolving these dependencies from the relational model
    // implies a large number of joins, which leads to a very slow query.

    included = R.map(
        item => {
            if(R.path(['data','attributes','subType'], item) === 'interactiveHTML'){
                return R.assocPath(
                        ['data', 'links','url'],
                        urlTransform(articleTuple[0].section, articleTuple[0].file_name, item.data.id),
                        item
                    );
            }
            return item;
        }
    , included);

    const primaryTag = R.path(['data', 'attributes', 'primaryTag'], article);

    const evolved = R.evolve({
        data: {
            attributes: R.merge(R.__, {
                socialHeadline,
                lastModifiedDate,
                checksum: lastModifiedDate, // to be removed
                leadMedia: resolveLeadMedia(leadMedia, tail),
                authors: deduplicateAuthors(tail.filter(x => x[0].type === 'author').map(R.last)),
                webview: isWebviewArticle(article, included),
                tags: evolveTags(primaryTag)(tail),
                searchTags: evolveSearchTags(tail)
            }),
            links: R.merge(R.__, {ownerUrl})
        },
        included: R.always(included)
    }, article);
    return evolved;
};

function resolveLeadMedia(leadMedia, tail) {
    let processedLeadMedia;

    if (!R.isNil(leadMedia) && leadMedia.noContent === true) {
        const staticContent = leadMedia.static_content || {};
        const findLeadMedia = R.find(tuple => tuple[0].id === leadMedia.id, tail);
        processedLeadMedia = findLeadMedia ? findLeadMedia[1] : null;

        if (!R.isNil(processedLeadMedia) && !R.isNil(R.path(['data', 'attributes'], processedLeadMedia))) {
            const attributes = R.merge(processedLeadMedia.data.attributes, staticContent);
            processedLeadMedia = R.assocPath(['data', 'attributes'], attributes, processedLeadMedia);
        }

    } else {
        processedLeadMedia = leadMedia;
    }

    if (!R.isNil(processedLeadMedia) && R.path(['data', 'type'], processedLeadMedia) === 'video') {
        return R.assocPath(['data', 'attributes', 'leadMedia'], resolveLeadMedia(R.defaultTo(null, processedLeadMedia.data.attributes.leadMedia), tail), processedLeadMedia);
    }

    return R.isNil(processedLeadMedia) ? processedLeadMedia : cropLeadImages(config, processedLeadMedia);
}

const cropLeadImages = R.curry(function(config, leadMedia) {
    if (R.path(['data', 'type'], leadMedia) === 'image') {
        const filteredLeadMedia = R.evolve({data: {attributes: {alternates: filterAlternates('leadMedia')}}}, leadMedia);
        return R.evolve({
            data: {
                attributes: R.flip(R.merge)(config.leadImageDimensions),
                links: {
                    url: R.replace(/ALTERNATES\/s\d{3}b?/, `ALTERNATES/${config.leadImageDimensions.code}`),
                    thumbnailUrl: R.replace(/ALTERNATES\/s\d{3}b?/, `ALTERNATES/${config.thumbnailDimensions.code}`)
                }
            }
        }, filteredLeadMedia);

    } else if (!R.isNil(R.path(['data', 'attributes', 'leadMedia'], leadMedia))) {

        return R.evolve({
            data: {
                attributes: {
                    leadMedia: cropLeadImages(config)
                }
            }
        }, leadMedia);

    }
    return leadMedia;
});

function createIncludedList(article, tail) {
    if (R.path(['data', 'type'], article) === 'live-event') {
        article.included = R.append({
            data: {
                type: 'liveEventEntries',
                links: {
                    url: article.data.links.liveEventEntriesUrl
                }
            }
        }, article.included);
    }

    return article.included.map(article_content => {
        if (article_content.noContent === true) {
            const staticContent = article_content.static_content || {};

            const tupleToSub = R.find(tuple => {
                return tuple[0].id === article_content.id;
            }, tail);

            if(!tupleToSub) {
                return null;
            }
            if (!R.isNil(R.path(['data', 'attributes'], tupleToSub[1]))) {
                return R.assocPath(['data', 'attributes'], R.merge(tupleToSub[1].data.attributes, staticContent), tupleToSub[1]);
            }
            return tupleToSub[1];
        }
        return article_content;
    }).filter(x => !R.isNil(x));
}

const formatGalleries = R.curry(function(config, tail) {
    return R.map(tuple => {
        const type = tuple[0].type;
        if (type === 'gallery') {
            return [tuple[0], R.evolve({
                data: {
                    attributes: {
                        leadMedia: cropLeadImages(config)
                    }
                },
                included: R.map(galleryImage => {
                    const filteredImage = R.evolve({data: {attributes: {alternates: filterAlternates('gallery')}}}, galleryImage);
                    return R.evolve({
                        data: {
                            attributes: R.flip(R.merge)(config.galleryImageDimensions),
                            links: {
                                url: R.replace(/ALTERNATES\/s\d{3}b?/, `ALTERNATES/${config.galleryImageDimensions.code}`)
                            }
                        }
                    }, filteredImage);
                })
            }, tuple[1])];
        }
        return tuple;
    }, tail);
});

function isWebviewArticle(article, included) {
    if (R.path(['data', 'attributes', 'webview'], article) === true) {
        return true;
    }
    return R.any(R.anyPass([
        R.pathEq(['data', 'type'], 'quiz'),
        R.pathEq(['data', 'type'], 'interactiveHtml')
    ]), included);
}

const filterAlternates = R.curry((context, alternates) => {
    if (R.isNil(alternates)) return null;
    let crops = [];
    switch(context) {
        case 'leadMedia':
            crops = R.filter(R.propSatisfies(R.test(/^.+[^b]$/), 'crop'), alternates);
            break;
        case 'gallery':
            crops = R.filter(R.propSatisfies(R.test(/^s1227b$/), 'crop'), alternates);
            break;
        case 'image':
            crops = R.filter(R.propSatisfies(R.test(/^.+b$/), 'crop'), alternates);
            break;
        default:
            return alternates;
    }
    return R.isEmpty(crops) ? null : crops;
});

function formatInlineImages(tail) {
    return R.map(tuple => {
        const type = tuple[0].type;
        if (type === 'image') {
            return [tuple[0], R.evolve({
                data: {
                    attributes: {
                        alternates: filterAlternates('image')
                    }
                }
            }, tuple[1])];
        }
        return tuple;
    }, tail);
}

function deduplicateAuthors(authorsList) {
    return R.reduce((mem, author) => {
        if (R.isNil(author.id) && R.filter(R.propEq('name', author.name), authorsList).length > 1) {
            return mem;
        }
        mem.push(author);
        return mem;
    }, [], authorsList);
}